# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("dsprop.dll")
prototypes = \
    {
        #
        'ADsPropCreateNotifyObj': SimTypeFunction([SimTypeBottom(label="IDataObject"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAppThdDataObj", "pwzADsObjName", "phNotifyObj"]),
        #
        'ADsPropGetInitInfo': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "hr": SimTypeInt(signed=True, label="Int32"), "pDsObj": SimTypeBottom(label="IDirectoryObject"), "pwzCN": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pWritableAttrs": SimTypePointer(SimStruct({"pszAttrName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwControlCode": SimTypeInt(signed=False, label="UInt32"), "dwADsType": SimTypeInt(signed=False, label="ADSTYPEENUM"), "pADsValues": SimTypePointer(SimStruct({"dwType": SimTypeInt(signed=False, label="ADSTYPEENUM"), "Anonymous": SimUnion({"DNString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseExactString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseIgnoreString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "PrintableString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "NumericString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "Boolean": SimTypeInt(signed=False, label="UInt32"), "Integer": SimTypeInt(signed=False, label="UInt32"), "OctetString": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_STRING", pack=False, align=None), "UTCTime": SimStruct({"wYear": SimTypeShort(signed=False, label="UInt16"), "wMonth": SimTypeShort(signed=False, label="UInt16"), "wDayOfWeek": SimTypeShort(signed=False, label="UInt16"), "wDay": SimTypeShort(signed=False, label="UInt16"), "wHour": SimTypeShort(signed=False, label="UInt16"), "wMinute": SimTypeShort(signed=False, label="UInt16"), "wSecond": SimTypeShort(signed=False, label="UInt16"), "wMilliseconds": SimTypeShort(signed=False, label="UInt16")}, name="SYSTEMTIME", pack=False, align=None), "LargeInteger": SimTypeBottom(label="LARGE_INTEGER"), "ClassName": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "ProviderSpecific": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_PROV_SPECIFIC", pack=False, align=None), "pCaseIgnoreList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_CASEIGNORE_LIST"), offset=0), "String": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_CASEIGNORE_LIST", pack=False, align=None), offset=0), "pOctetList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_OCTET_LIST"), offset=0), "Length": SimTypeInt(signed=False, label="UInt32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_LIST", pack=False, align=None), offset=0), "pPath": SimTypePointer(SimStruct({"Type": SimTypeInt(signed=False, label="UInt32"), "VolumeName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Path": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_PATH", pack=False, align=None), offset=0), "pPostalAddress": SimTypePointer(SimStruct({"PostalAddress": SimTypeFixedSizeArray(SimTypePointer(SimTypeChar(label="Char"), offset=0), 6)}, name="ADS_POSTALADDRESS", pack=False, align=None), offset=0), "Timestamp": SimStruct({"WholeSeconds": SimTypeInt(signed=False, label="UInt32"), "EventID": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TIMESTAMP", pack=False, align=None), "BackLink": SimStruct({"RemoteID": SimTypeInt(signed=False, label="UInt32"), "ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_BACKLINK", pack=False, align=None), "pTypedName": SimTypePointer(SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Level": SimTypeInt(signed=False, label="UInt32"), "Interval": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TYPEDNAME", pack=False, align=None), offset=0), "Hold": SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Amount": SimTypeInt(signed=False, label="UInt32")}, name="ADS_HOLD", pack=False, align=None), "pNetAddress": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0), "pReplicaPointer": SimTypePointer(SimStruct({"ServerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ReplicaType": SimTypeInt(signed=False, label="UInt32"), "ReplicaNumber": SimTypeInt(signed=False, label="UInt32"), "Count": SimTypeInt(signed=False, label="UInt32"), "ReplicaAddressHints": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0)}, name="ADS_REPLICAPOINTER", pack=False, align=None), offset=0), "pFaxNumber": SimTypePointer(SimStruct({"TelephoneNumber": SimTypePointer(SimTypeChar(label="Char"), offset=0), "NumberOfBits": SimTypeInt(signed=False, label="UInt32"), "Parameters": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_FAXNUMBER", pack=False, align=None), offset=0), "Email": SimStruct({"Address": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Type": SimTypeInt(signed=False, label="UInt32")}, name="ADS_EMAIL", pack=False, align=None), "SecurityDescriptor": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NT_SECURITY_DESCRIPTOR", pack=False, align=None), "pDNWithBinary": SimTypePointer(SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpBinaryValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_BINARY", pack=False, align=None), offset=0), "pDNWithString": SimTypePointer(SimStruct({"pszStringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_STRING", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="ADSVALUE", pack=False, align=None), offset=0), "dwNumValues": SimTypeInt(signed=False, label="UInt32")}, name="ADS_ATTR_INFO", pack=False, align=None), offset=0)}, name="ADSPROPINITPARAMS", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hNotifyObj", "pInitParams"]),
        #
        'ADsPropSetHwndWithTitle': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="SByte"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hNotifyObj", "hPage", "ptzTitle"]),
        #
        'ADsPropSetHwnd': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hNotifyObj", "hPage"]),
        #
        'ADsPropCheckIfWritable': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"pszAttrName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwControlCode": SimTypeInt(signed=False, label="UInt32"), "dwADsType": SimTypeInt(signed=False, label="ADSTYPEENUM"), "pADsValues": SimTypePointer(SimStruct({"dwType": SimTypeInt(signed=False, label="ADSTYPEENUM"), "Anonymous": SimUnion({"DNString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseExactString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseIgnoreString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "PrintableString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "NumericString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "Boolean": SimTypeInt(signed=False, label="UInt32"), "Integer": SimTypeInt(signed=False, label="UInt32"), "OctetString": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_STRING", pack=False, align=None), "UTCTime": SimStruct({"wYear": SimTypeShort(signed=False, label="UInt16"), "wMonth": SimTypeShort(signed=False, label="UInt16"), "wDayOfWeek": SimTypeShort(signed=False, label="UInt16"), "wDay": SimTypeShort(signed=False, label="UInt16"), "wHour": SimTypeShort(signed=False, label="UInt16"), "wMinute": SimTypeShort(signed=False, label="UInt16"), "wSecond": SimTypeShort(signed=False, label="UInt16"), "wMilliseconds": SimTypeShort(signed=False, label="UInt16")}, name="SYSTEMTIME", pack=False, align=None), "LargeInteger": SimTypeBottom(label="LARGE_INTEGER"), "ClassName": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "ProviderSpecific": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_PROV_SPECIFIC", pack=False, align=None), "pCaseIgnoreList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_CASEIGNORE_LIST"), offset=0), "String": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_CASEIGNORE_LIST", pack=False, align=None), offset=0), "pOctetList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_OCTET_LIST"), offset=0), "Length": SimTypeInt(signed=False, label="UInt32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_LIST", pack=False, align=None), offset=0), "pPath": SimTypePointer(SimStruct({"Type": SimTypeInt(signed=False, label="UInt32"), "VolumeName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Path": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_PATH", pack=False, align=None), offset=0), "pPostalAddress": SimTypePointer(SimStruct({"PostalAddress": SimTypeFixedSizeArray(SimTypePointer(SimTypeChar(label="Char"), offset=0), 6)}, name="ADS_POSTALADDRESS", pack=False, align=None), offset=0), "Timestamp": SimStruct({"WholeSeconds": SimTypeInt(signed=False, label="UInt32"), "EventID": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TIMESTAMP", pack=False, align=None), "BackLink": SimStruct({"RemoteID": SimTypeInt(signed=False, label="UInt32"), "ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_BACKLINK", pack=False, align=None), "pTypedName": SimTypePointer(SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Level": SimTypeInt(signed=False, label="UInt32"), "Interval": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TYPEDNAME", pack=False, align=None), offset=0), "Hold": SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Amount": SimTypeInt(signed=False, label="UInt32")}, name="ADS_HOLD", pack=False, align=None), "pNetAddress": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0), "pReplicaPointer": SimTypePointer(SimStruct({"ServerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ReplicaType": SimTypeInt(signed=False, label="UInt32"), "ReplicaNumber": SimTypeInt(signed=False, label="UInt32"), "Count": SimTypeInt(signed=False, label="UInt32"), "ReplicaAddressHints": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0)}, name="ADS_REPLICAPOINTER", pack=False, align=None), offset=0), "pFaxNumber": SimTypePointer(SimStruct({"TelephoneNumber": SimTypePointer(SimTypeChar(label="Char"), offset=0), "NumberOfBits": SimTypeInt(signed=False, label="UInt32"), "Parameters": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_FAXNUMBER", pack=False, align=None), offset=0), "Email": SimStruct({"Address": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Type": SimTypeInt(signed=False, label="UInt32")}, name="ADS_EMAIL", pack=False, align=None), "SecurityDescriptor": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NT_SECURITY_DESCRIPTOR", pack=False, align=None), "pDNWithBinary": SimTypePointer(SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpBinaryValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_BINARY", pack=False, align=None), offset=0), "pDNWithString": SimTypePointer(SimStruct({"pszStringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_STRING", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="ADSVALUE", pack=False, align=None), offset=0), "dwNumValues": SimTypeInt(signed=False, label="UInt32")}, name="ADS_ATTR_INFO", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzAttr", "pWritableAttrs"]),
        #
        'ADsPropSendErrorMessage': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"hwndPage": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszPageTitle": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszObjPath": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszObjClass": SimTypePointer(SimTypeChar(label="Char"), offset=0), "hr": SimTypeInt(signed=True, label="Int32"), "pszError": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADSPROPERROR", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hNotifyObj", "pError"]),
        #
        'ADsPropShowErrorDialog': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hNotifyObj", "hPage"]),
    }

lib.set_prototypes(prototypes)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.exploration_techniques"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestLoopSeer(unittest.TestCase):
    def test_various_loops(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "various_loops"), auto_load_libs=False)

        cfg = p.analyses.CFGFast(normalize=True)

        state = p.factory.entry_state()
        state.register_plugin("loop_data", angr.state_plugins.SimStateLoopData())

        dummy = p.loader.main_object.get_symbol("dummy")
        bvs = state.solver.BVS(dummy.name, 8 * dummy.size)
        state.memory.store(dummy.rebased_addr, bvs, endness="Iend_LE")

        simgr = p.factory.simulation_manager(state)

        simgr.use_technique(angr.exploration_techniques.LoopSeer(cfg=cfg, functions=None, bound=None))

        simgr.run()

        assert len(simgr.deadended) == 10
        assert len(simgr.deadended[0].loop_data.back_edge_trip_counts) == 14

        for i, d in enumerate(simgr.deadended):
            f = p.kb.functions.function(name="symbolic_loop")
            l = p.analyses.LoopFinder(functions=[f]).loops[0]
            assert d.loop_data.back_edge_trip_counts[l.entry.addr][0] == i

            f = p.kb.functions.function(name="for_loop")
            l = p.analyses.LoopFinder(functions=[f]).loops[0]
            assert d.loop_data.back_edge_trip_counts[l.entry.addr][0] == 9

            f = p.kb.functions.function(name="while_loop")
            l = p.analyses.LoopFinder(functions=[f]).loops[0]
            assert d.loop_data.back_edge_trip_counts[l.entry.addr][0] == 9

            f = p.kb.functions.function(name="do_while_loop")
            l = p.analyses.LoopFinder(functions=[f]).loops[0]
            assert d.loop_data.header_trip_counts[l.entry.addr][0] == 9

            f = p.kb.functions.function(name="nullify")
            l = p.analyses.LoopFinder(functions=[f]).loops[0]
            assert len(d.loop_data.back_edge_trip_counts[l.entry.addr]) == 8
            assert d.loop_data.back_edge_trip_counts[l.entry.addr][0] == 9

            f = p.kb.functions.function(name="nested_for_loop")
            ol = p.analyses.LoopFinder(functions=[f]).loops[0]
            il = ol.subloops[0]
            assert d.loop_data.back_edge_trip_counts[ol.entry.addr][0] == 3
            assert len(d.loop_data.back_edge_trip_counts[il.entry.addr]) == 3
            assert all(s == 3 for s in d.loop_data.back_edge_trip_counts[il.entry.addr])

            f = p.kb.functions.function(name="nested_while_loop")
            ol = p.analyses.LoopFinder(functions=[f]).loops[0]
            il = ol.subloops[0]
            assert d.loop_data.back_edge_trip_counts[ol.entry.addr][0] == 3
            assert len(d.loop_data.back_edge_trip_counts[il.entry.addr]) == 3
            assert all(s == 3 for s in d.loop_data.back_edge_trip_counts[il.entry.addr])

            f = p.kb.functions.function(name="nested_do_while_loop")
            ol = p.analyses.LoopFinder(functions=[f]).loops[0]
            il = ol.subloops[0]
            assert d.loop_data.header_trip_counts[ol.entry.addr][0] == 3
            assert len(d.loop_data.header_trip_counts[il.entry.addr]) == 3
            assert all(s == 3 for s in d.loop_data.header_trip_counts[il.entry.addr])

            f = p.kb.functions.function(name="break_for_loop")
            l = p.analyses.LoopFinder(functions=[f]).loops[0]
            assert d.loop_data.back_edge_trip_counts[l.entry.addr][0] == 9

            f = p.kb.functions.function(name="break_do_while_loop")
            l = p.analyses.LoopFinder(functions=[f]).loops[0]
            assert d.loop_data.header_trip_counts[l.entry.addr][0] == 9

    def test_loops_with_invalid_parameter(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "test_loops"), auto_load_libs=False)

        state = p.factory.entry_state()
        state.register_plugin("loop_data", angr.state_plugins.SimStateLoopData())
        simgr = p.factory.simulation_manager(state)

        simgr.use_technique(angr.exploration_techniques.LoopSeer(functions=["main", 0x1234], bound=None))

        simgr.run()

        assert len(simgr.deadended[0].loop_data.back_edge_trip_counts) == 3
        assert simgr.deadended[0].loop_data.back_edge_trip_counts[0x400665][0] == 10
        assert len(simgr.deadended[0].loop_data.back_edge_trip_counts[0x400665]) == 10
        assert simgr.deadended[0].loop_data.back_edge_trip_counts[0x400675][0] == 10
        assert simgr.deadended[0].loop_data.back_edge_trip_counts[0x4006B2][0] == 100

    def test_arrays(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "test_arrays"), auto_load_libs=False)

        cfg = p.analyses.CFGFast(normalize=True)

        state = p.factory.entry_state()
        state.register_plugin("loop_data", angr.state_plugins.SimStateLoopData())
        simgr = p.factory.simulation_manager(state)

        simgr.use_technique(angr.exploration_techniques.LoopSeer(cfg=cfg, functions="main", bound=None))

        simgr.run()

        assert len(simgr.deadended[0].loop_data.back_edge_trip_counts) == 2
        assert simgr.deadended[0].loop_data.back_edge_trip_counts[0x400636][0] == 26
        assert simgr.deadended[0].loop_data.back_edge_trip_counts[0x4005FD][0] == 26

    def test_loop_limiter(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "test_arrays"), auto_load_libs=False)

        cfg = p.analyses.CFGFast(normalize=True)

        state = p.factory.entry_state()
        state.register_plugin("loop_data", angr.state_plugins.SimStateLoopData())
        simgr = p.factory.simulation_manager(state)

        simgr.use_technique(angr.exploration_techniques.LoopSeer(cfg=cfg, functions="main", bound=5))

        simgr.run()

        assert "spinning" in simgr.stashes
        assert simgr.spinning[0].loop_data.back_edge_trip_counts[0x4005FD][0] == 6

    def test_loop_limiter_constant_loop(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "constant_loopseer"), auto_load_libs=False)

        cfg = p.analyses.CFGFast(normalize=True)

        state = p.factory.entry_state()
        simgr = p.factory.simulation_manager(state)

        simgr.use_technique(
            angr.exploration_techniques.LoopSeer(cfg=cfg, functions="main", bound=5, limit_concrete_loops=False)
        )

        simgr.run()
        assert simgr.deadended[0].regs.eax.concrete
        val = simgr.deadended[0].solver.eval_one(simgr.deadended[0].regs.eax)
        assert val == 420


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses.cfg"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

try:
    import pysoot
except ModuleNotFoundError:
    pysoot = None

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
@unittest.skipUnless(pysoot, "pysoot not available")
class TestCfgfastSoot(unittest.TestCase):
    def test_simple1(self):
        binary_path = os.path.join(test_location, "java", "simple1.jar")
        p = angr.Project(binary_path, main_opts={"entry_point": "simple1.Class1.main"}, auto_load_libs=False)
        cfg = p.analyses.CFGFastSoot()
        assert cfg.graph.nodes()

    def test_simple2(self):
        binary_path = os.path.join(test_location, "java", "simple2.jar")
        p = angr.Project(binary_path, main_opts={"entry_point": "simple2.Class1.main"}, auto_load_libs=False)
        cfg = p.analyses.CFGFastSoot()
        assert cfg.graph.nodes()


if __name__ == "__main__":
    unittest.main()



import os
import time

import angr
from angr import options as so

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../../../binaries")
p = angr.Project(os.path.join(test_location, "tests", "x86_64", "perf_unicorn_1"))
s_unicorn = p.factory.entry_state(
    add_options=so.unicorn | {so.STRICT_PAGE_ACCESS}, remove_options={so.LAZY_SOLVES}
)  # unicorn


def main():
    sm_unicorn = p.factory.simulation_manager(s_unicorn)
    sm_unicorn.run()


if __name__ == "__main__":
    tstart = time.time()
    main()
    tend = time.time()
    print("Elapsed: %f sec" % (tend - tstart))



import logging
import copy
from collections import defaultdict

from .plugin import SimStatePlugin


l = logging.getLogger(name=__name__)


class SimStateLoopData(SimStatePlugin):
    """
    This class keeps track of loop-related information for states.
    Note that we have 2 counters for loop iterations (trip counts): the first
    recording the number of times one of the back edges (or continue edges) of
    a loop is taken, whereas the second recording the number of times the loop
    header (or loop entry) is executed. These 2 counters may differ since
    compilers usually optimize loops hence completely change the loop structure
    at the binary level.
    This is supposed to be used with `LoopSeer` exploration technique, which
    monitors loop execution. For the moment, the only thing we want to analyze
    is loop trip counts, but nothing prevents us from extending this plugin for
    other loop analyses.
    """

    def __init__(self, back_edge_trip_counts=None, header_trip_counts=None, current_loop=None):
        """
        :param back_edge_trip_counts: Dictionary that stores back edge based trip counts for each loop.
                                      Keys are address of loop headers.
        :param header_trip_counts:    Dictionary that stores header based trip counts for each loop.
                                      Keys are address of loop headers.
        :param current_loop:          List of currently running loops. Each element is a tuple
                                      (loop object, list of loop exits).
        """
        # This is why header based trip counter is not always accurate:
        #
        # 0x10812: movs r3, #0          -> this block dominates the loop
        # 0x10814: str  r3, [r7, #20]
        # 0x10816: b    0x10868
        #
        # 0x10818: movs r3, #0          -> the real loop body starts here
        # ...
        #
        # 0x10868: ldr  r3, [r7, #20]   -> the loop header is executed the first time without executing the loop body
        # 0x1086a: cmp  r3, #3
        # 0x1086c: ble  0x10818         -> the back edge
        #
        # This is why back edge based trip counter is not always accurate. The
        # compiler divides the logic of the loop body in 2 parts A and B. This loop
        # structure implies that the back edge is taken one time less than the number
        # of times the full loop body is executed.
        #
        # 0x10768: movs r3, #0          -> this block contains part A of the logic
        # ...                              for the first iteration and dominates the loop
        # 0x10816: b    0x10868
        #
        # 0x10818: movs r3, #0          -> part A of the logic for other iterations is put in this block
        # ...
        #
        # 0x10868: add  r2, r0, r1      -> part B of the logic for all iterations is put in the loop header
        # ...
        # 0x10898: ldr  r3, [r7, #20]
        # 0x1089a: cmp  r3, #3
        # 0x1089c: ble  0x10818         -> the back edge
        #
        # And yes, another example for the latter case is a do-while loop!

        SimStatePlugin.__init__(self)
        self.back_edge_trip_counts = defaultdict(list) if back_edge_trip_counts is None else back_edge_trip_counts
        self.header_trip_counts = defaultdict(list) if header_trip_counts is None else header_trip_counts
        self.current_loop = [] if current_loop is None else current_loop

    def merge(self, others, merge_conditions, common_ancestor=None):  # pylint: disable=unused-argument
        l.warning("Merging is not implemented for loop data!")
        return False

    def widen(self, others):  # pylint: disable=unused-argument
        l.warning("Widening is not implemented for loop data!")
        return False

    @SimStatePlugin.memo
    def copy(self, memo):  # pylint: disable=unused-argument
        return SimStateLoopData(
            back_edge_trip_counts=copy.deepcopy(self.back_edge_trip_counts),
            header_trip_counts=copy.deepcopy(self.header_trip_counts),
            current_loop=list(self.current_loop),
        )


from angr.sim_state import SimState

SimState.register_default("loop_data", SimStateLoopData)



import logging
from . import SimConcretizationStrategy


class SimConcretizationStrategyLogging(SimConcretizationStrategy):
    """
    Concretization strategy that logs concretization results from another strategy.
    """

    def __init__(self, strategy: SimConcretizationStrategy, is_read_strategy: bool):
        super().__init__()
        self._strategy = strategy
        self._is_read_strategy = is_read_strategy

    def _concretize(self, memory, addr, **kwargs):
        answers = self._strategy._concretize(memory, addr, **kwargs)
        if answers is not None:
            if self._is_read_strategy:
                logging.debug(
                    "Read strategy %s on %s gave [%s]",
                    type(self._strategy).__name__,
                    addr,
                    ", ".join([hex(answer) for answer in answers]),
                )
            else:
                logging.debug(
                    "Write strategy %s on %s gave [%s]",
                    type(self._strategy).__name__,
                    addr,
                    ", ".join([hex(answer) for answer in answers]),
                )
        return answers



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("api-ms-win-core-memory-l1-1-3.dll")
prototypes = \
    {
        #
        'SetProcessValidCallTargets': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Offset": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Flags": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="CFG_CALL_TARGET_INFO", pack=False, align=None), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hProcess", "VirtualAddress", "RegionSize", "NumberOfOffsets", "OffsetInformation"]),
        #
        'VirtualAllocFromApp': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="VIRTUAL_ALLOCATION_TYPE"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["BaseAddress", "Size", "AllocationType", "Protection"]),
        #
        'VirtualProtectFromApp': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Address", "Size", "NewProtection", "OldProtection"]),
        #
        'OpenFileMappingFromApp': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["DesiredAccess", "InheritHandle", "Name"]),
    }

lib.set_prototypes(prototypes)



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("compstui.dll")
prototypes = \
    {
        #
        'CommonPropertySheetUIA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"cbSize": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16"), "Reason": SimTypeShort(signed=False, label="UInt16"), "hComPropSheet": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pfnComPropSheet": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hComPropSheet", "Function", "lParam1", "lParam2"]), offset=0), "lParamInit": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "UserData": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Result": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="PROPSHEETUI_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPSUIInfo", "lParam"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWndOwner", "pfnPropSheetUI", "lParam", "pResult"]),
        #
        'CommonPropertySheetUIW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"cbSize": SimTypeShort(signed=False, label="UInt16"), "Version": SimTypeShort(signed=False, label="UInt16"), "Flags": SimTypeShort(signed=False, label="UInt16"), "Reason": SimTypeShort(signed=False, label="UInt16"), "hComPropSheet": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pfnComPropSheet": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hComPropSheet", "Function", "lParam1", "lParam2"]), offset=0), "lParamInit": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "UserData": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "Result": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)}, name="PROPSHEETUI_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pPSUIInfo", "lParam"]), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWndOwner", "pfnPropSheetUI", "lParam", "pResult"]),
        #
        'GetCPSUIUserData': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), arg_names=["hDlg"]),
        #
        'SetCPSUIUserData': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hDlg", "CPSUIUserData"]),
    }

lib.set_prototypes(prototypes)



import os
import time

import angr

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../../../binaries")
p = angr.Project(os.path.join(test_location, "tests", "x86_64", "true"), auto_load_libs=False)
funcs = list(p.analyses.CFGFast().functions.keys())


def main():
    p.analyses.CFGEmulated(starts=funcs, call_depth=0)


if __name__ == "__main__":
    tstart = time.time()
    main()
    tend = time.time()
    print("Elapsed: %f sec" % (tend - tstart))



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long

import unittest

import archinfo

import angr
from angr.analyses.cfg import CFGBase


class TestNoopBlocks(unittest.TestCase):
    def test_x86_noop_blocks(self):
        # nop
        arch = archinfo.arch_from_id("x86")
        b = b"\x90\x90\x90\x90\x90\x90\x90\x90"
        p = angr.load_shellcode(b, arch, load_address=0x400000)
        block = p.factory.block(0x400000, opt_level=1, cross_insn_opt=False)
        assert CFGBase._is_noop_block(arch, block) is True
        block = p.factory.block(0x400000, opt_level=1, cross_insn_opt=True)
        assert CFGBase._is_noop_block(arch, block) is True

    def test_amd64_noop_blocks(self):
        # nop
        arch = archinfo.arch_from_id("amd64")
        b = b"\x90\x90\x90\x90\x90\x90\x90\x90"
        p = angr.load_shellcode(b, arch, load_address=0x400000)
        block = p.factory.block(0x400000, opt_level=1, cross_insn_opt=False)
        assert CFGBase._is_noop_block(arch, block) is True
        block = p.factory.block(0x400000, opt_level=1, cross_insn_opt=True)
        assert CFGBase._is_noop_block(arch, block) is True

    def test_arm_noop_blocks(self):
        arch = archinfo.arch_from_id("ARMEL")

        # andeq r0, r0, r0
        b = b"\x00\x00\x00\x00\x00\x00\x00\x00"
        p = angr.load_shellcode(b, arch, load_address=0x400000)
        block = p.factory.block(0x400000, opt_level=1, cross_insn_opt=False)
        assert CFGBase._is_noop_block(arch, block) is True
        block = p.factory.block(0x400000, opt_level=1, cross_insn_opt=True)
        assert CFGBase._is_noop_block(arch, block) is True

        # mov r0, r0
        b = b"\x00\x00\xa0\xe1"
        p = angr.load_shellcode(b, arch, load_address=0x400000)
        block = p.factory.block(0x400000, opt_level=1, cross_insn_opt=False)
        assert CFGBase._is_noop_block(arch, block) is True
        block = p.factory.block(0x400000, opt_level=1, cross_insn_opt=True)
        assert CFGBase._is_noop_block(arch, block) is True


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint:disable=missing-class-docstring,no-self-use
__package__ = __package__ or "tests.procedures.posix"  # pylint:disable=redefined-builtin

import os
import unittest

import angr
from angr.state_plugins.posix import Flags

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestChroot(unittest.TestCase):
    def test_chroot(self):
        project = angr.Project(os.path.join(test_location, "x86_64", "chroot_test"))
        initial_state = project.factory.entry_state()

        simgr = project.factory.simgr(initial_state)

        simgr.run()

        # Try and read the files stat size from new chrooted dir
        fd = simgr.deadended[0].posix.open("/test.txt", Flags.O_RDONLY)
        stat = simgr.deadended[0].posix.fstat(fd)
        print(f"File's Stat Size: {stat.st_size}")


if __name__ == "__main__":
    unittest.main()



import claripy
import logging

from ..java import JavaSimProcedure
from ...engines.soot.expressions import SimSootExpr_NewArray, SimSootExpr_NullConstant
from ...engines.soot.values import SimSootValue_StringRef, SimSootValue_ThisRef
from .collection import ELEMS, SIZE

log = logging.getLogger(name=__name__)

MAP_SIZE = "size"
MAP_KEYS = "keys"


def get_map_key(state, key_ref):
    if isinstance(key_ref, SimSootValue_StringRef):
        string = state.memory.load(key_ref)
        if string.concrete:
            return state.solver.eval(string)
        else:
            return state.solver.eval(string)

    else:
        return key_ref.id


class MapInit(JavaSimProcedure):
    __provides__ = (("java.util.Map", "<init>()"), ("java.util.HashMap", "<init>()"))

    def run(self, this_ref):
        log.debug(f"Called SimProcedure java.util.Map.<init> with args: {this_ref}")
        # init map size
        this_ref.store_field(self.state, MAP_SIZE, "int", claripy.BVV(0, 32))
        array_ref = SimSootExpr_NewArray.new_array(self.state, "java.lang.Object", claripy.BVV(1000, 32))
        this_ref.store_field(self.state, MAP_KEYS, "java.lang.Object[]", array_ref)

        return


class MapPut(JavaSimProcedure):
    __provides__ = (
        ("java.util.Map", "put(java.lang.Object,java.lang.Object)"),
        ("java.util.HashMap", "put(java.lang.Object,java.lang.Object)"),
    )

    def run(self, this_ref, key_ref, value_ref):
        log.debug(f"Called SimProcedure java.util.Map.add with args: {this_ref} {key_ref} {value_ref}")

        if this_ref.symbolic:
            return SimSootExpr_NullConstant

        # store value
        this_ref.store_field(self.state, get_map_key(self.state, key_ref), "java.lang.Object", value_ref)
        # store key
        map_size = this_ref.load_field(self.state, MAP_SIZE, "int")
        keys_array_ref = this_ref.load_field(self.state, MAP_KEYS, "java.lang.Object[]")
        # TODO we should check if key_ref is already in keys_array_ref
        self.state.javavm_memory.store_array_element(keys_array_ref, map_size, key_ref)

        # Update size
        new_map_size = claripy.BVV(self.state.solver.eval(map_size) + 1, 32)
        this_ref.store_field(self.state, MAP_SIZE, "int", new_map_size)

        # TODO
        # returns the previous value associated with key, or null if there was no mapping for key.
        return SimSootExpr_NullConstant


class MapGet(JavaSimProcedure):
    __provides__ = (("java.util.Map", "get(java.lang.Object)"), ("java.util.HashMap", "get(java.lang.Object)"))

    def run(self, this_ref, key_ref):
        log.debug(f"Called SimProcedure java.util.Map.get with args: {this_ref} {key_ref}")

        if this_ref.symbolic:
            return SimSootValue_ThisRef(self.state, "java.lang.Object", symbolic=True)

        try:
            return this_ref.load_field(self.state, get_map_key(self.state, key_ref), "java.lang.Object")
        except (KeyError, AttributeError):
            return SimSootExpr_NullConstant


class MapSize(JavaSimProcedure):
    __provides__ = (("java.util.Map", "size()"), ("java.util.HashMap", "size()"))

    def run(self, this_ref):
        log.debug(f"Called SimProcedure java.util.Map.size with args: {this_ref}")

        if this_ref.symbolic:
            return claripy.BVS("map_size", 32)

        return this_ref.get_field(self.state, MAP_SIZE, "int")


class MapContainsKey(JavaSimProcedure):
    __provides__ = (
        ("java.util.Map", "containsKey(java.lang.Object)"),
        ("java.util.HashMap", "containsKey(java.lang.Object)"),
    )

    def run(self, this_ref, key_ref):
        log.debug(f"Called SimProcedure java.util.Map.containsKey with args: {this_ref} {key_ref}")

        if this_ref.symbolic:
            return claripy.BoolS("contains_key")

        try:
            this_ref.load_field(self.state, get_map_key(self.state, key_ref), "java.lang.Object")
            return claripy.BoolV(1)

        except (KeyError, AttributeError):
            return claripy.BoolV(0)


class MapKeySet(JavaSimProcedure):
    __provides__ = (("java.util.Map", "keySet()"), ("java.util.HashMap", "keySet()"))

    def run(self, this_ref):
        log.debug(f"Called SimProcedure java.util.Map.keySet with args: {this_ref}")

        if this_ref.symbolic:
            return SimSootValue_ThisRef.new_object(self.state, "java.util.Set", symbolic=True)

        set_ref = SimSootValue_ThisRef.new_object(self.state, "java.util.Set")
        keys_array_ref = this_ref.load_field(self.state, MAP_KEYS, "java.lang.Object[]")
        set_ref.store_field(self.state, ELEMS, "java.lang.Object[]", keys_array_ref)
        map_size = this_ref.load_field(self.state, MAP_SIZE, "int")
        set_size = claripy.BVV(self.state.solver.eval(map_size), 32)
        set_ref.store_field(self.state, SIZE, "int", set_size)

        return set_ref



import logging
import os

l = logging.getLogger(name=__name__)

from ..misc import autoimport
from ..sim_procedure import SimProcedure

# Import all classes under the current directory, and group them based on
# lib names.
SIM_PROCEDURES = {}
path = os.path.dirname(os.path.abspath(__file__))
skip_dirs = ["definitions"]

for pkg_name, package in autoimport.auto_import_packages("angr.procedures", path, skip_dirs):
    for _, mod in autoimport.filter_module(package, type_req=type(os)):
        for name, proc in autoimport.filter_module(mod, type_req=type, subclass_req=SimProcedure):
            if hasattr(proc, "__provides__"):
                for custom_pkg_name, custom_func_name in proc.__provides__:
                    if custom_pkg_name not in SIM_PROCEDURES:
                        SIM_PROCEDURES[custom_pkg_name] = {}
                    SIM_PROCEDURES[custom_pkg_name][custom_func_name] = proc
            else:
                if pkg_name not in SIM_PROCEDURES:
                    SIM_PROCEDURES[pkg_name] = {}
                SIM_PROCEDURES[pkg_name][name] = proc
                if hasattr(proc, "ALT_NAMES") and proc.ALT_NAMES:
                    for altname in proc.ALT_NAMES:
                        SIM_PROCEDURES[pkg_name][altname] = proc
                if name == "UnresolvableJumpTarget":
                    SIM_PROCEDURES[pkg_name]["UnresolvableTarget"] = proc


class _SimProcedures:
    def __getitem__(self, k):
        l.critical(
            "the SimProcedures dictionary is DEPRECATED. Please use the angr.SIM_PROCEDURES global dict instead."
        )
        return SIM_PROCEDURES[k]

    def __setitem__(self, k, v):
        l.critical(
            "the SimProcedures dictionary is DEPRECATED. Please use the angr.SIM_PROCEDURES global dict instead."
        )
        SIM_PROCEDURES[k] = v


SimProcedures = _SimProcedures()



# pylint:disable=arguments-differ
import logging
from typing import List, Set, Optional, Tuple, Union, Any, Iterable

from sortedcontainers import SortedDict

import claripy

from .....errors import SimMemoryError
from . import PageBase
from .cooperation import MemoryObjectMixin, SimMemoryObject


l = logging.getLogger(name=__name__)


class UltraPage(MemoryObjectMixin, PageBase):
    """
    Default page implementation
    """

    SUPPORTS_CONCRETE_LOAD = True

    def __init__(self, memory=None, init_zero=False, **kwargs):
        super().__init__(**kwargs)

        if memory is not None:
            self.concrete_data = bytearray(memory.page_size)
            if init_zero:
                self.symbolic_bitmap = bytearray(memory.page_size)
            else:
                self.symbolic_bitmap = bytearray(b"\1" * memory.page_size)
        else:
            self.concrete_data = None
            self.symbolic_bitmap = None

        self.symbolic_data = SortedDict()

    @classmethod
    def new_from_shared(cls, data, memory=None, **kwargs):
        o = cls(**kwargs)
        o.concrete_data = data
        o.symbolic_bitmap = bytearray(memory.page_size)
        o.refcount = 2  # pylint: disable=attribute-defined-outside-init
        return o

    def copy(self, memo):
        o = super().copy(memo)
        o.concrete_data = bytearray(self.concrete_data)
        o.symbolic_bitmap = bytearray(self.symbolic_bitmap)
        o.symbolic_data = SortedDict(self.symbolic_data)
        return o

    def load(
        self, addr, size=None, page_addr=None, endness=None, memory=None, cooperate=False, **kwargs
    ):  # pylint: disable=arguments-differ
        concrete_run = []
        symbolic_run = ...
        last_run = None
        result = []

        def cycle(end):
            if last_run is symbolic_run and symbolic_run is None:
                fill(end)
            elif last_run is concrete_run:
                new_ast = claripy.BVV(concrete_run, (end - result[-1][0]) * memory.state.arch.byte_width)
                new_obj = SimMemoryObject(new_ast, result[-1][0], endness)
                result[-1] = (result[-1][0], new_obj)

        def fill(end):
            global_end_addr = end
            global_start_addr = result[-1][0]
            size = global_end_addr - global_start_addr
            new_ast = self._default_value(
                global_start_addr,
                size,  # pylint: disable=assignment-from-no-return
                key=(self.category, global_start_addr),
                memory=memory,
                endness=endness,
                **kwargs,
            )
            new_item = SimMemoryObject(new_ast, global_start_addr, endness=endness)
            self.symbolic_data[global_start_addr - page_addr] = new_item
            result[-1] = (global_start_addr, new_item)

        for subaddr in range(addr, addr + size):
            realaddr = subaddr + page_addr
            if self.symbolic_bitmap[subaddr]:
                cur_val = self._get_object(subaddr, page_addr, memory=memory)
                if cur_val is last_run and last_run is symbolic_run:
                    pass
                else:
                    cycle(realaddr)
                    last_run = symbolic_run = cur_val
                    result.append((realaddr, cur_val))
            else:
                cur_val = self.concrete_data[subaddr]
                if last_run is concrete_run:
                    if endness == "Iend_LE":
                        last_run = concrete_run = concrete_run | (
                            cur_val << (memory.state.arch.byte_width * (realaddr - result[-1][0]))
                        )
                    else:
                        last_run = concrete_run = (concrete_run << memory.state.arch.byte_width) | cur_val
                    result[-1] = (result[-1][0], concrete_run)
                else:
                    cycle(realaddr)
                    last_run = concrete_run = cur_val
                    result.append((realaddr, cur_val))

        cycle(page_addr + addr + size)
        if not cooperate:
            result = self._force_load_cooperation(result, size, endness, page_addr=page_addr, memory=memory, **kwargs)
        return result

    def store(
        self,
        addr,
        data: Union[int, SimMemoryObject],
        size: int = None,
        endness=None,
        memory=None,
        page_addr=None,  # pylint: disable=arguments-differ
        cooperate=False,
        **kwargs,
    ):
        super().store(
            addr, data, size=size, endness=endness, memory=memory, cooperate=cooperate, page_addr=page_addr, **kwargs
        )

        if not cooperate:
            data = self._force_store_cooperation(
                addr, data, size, endness, page_addr=page_addr, memory=memory, **kwargs
            )

        if size >= memory.page_size - addr:
            size = memory.page_size - addr

        if type(data) is not int:
            if data.object.op == "BVV" and not data.object.annotations:
                # trim the unnecessary leading bytes if there are any
                full_bits = len(data.object)
                start = (page_addr + addr - data.base) & ((1 << memory.state.arch.bits) - 1)
                if start >= data.base + data.length:
                    raise SimMemoryError("Not enough bytes to store.")
                start_bits = full_bits - start * memory.state.arch.byte_width - 1
                # trim the overflowing bytes if there are any
                end_bits = start_bits + 1 - size * memory.state.arch.byte_width
                if start_bits != full_bits - 1 or end_bits != 0:
                    if endness == "Iend_LE":
                        start_bits, end_bits = len(data.object) - end_bits - 1, len(data.object) - start_bits - 1
                    obj = data.object[start_bits:end_bits]
                    data = obj.args[0]

        if type(data) is int or (data.object.op == "BVV" and not data.object.annotations):
            # mark range as not symbolic
            self.symbolic_bitmap[addr : addr + size] = b"\0" * size

            # store
            arange = range(addr, addr + size)
            if type(data) is int:
                ival = data
            else:  # data.object.op == 'BVV'
                ival = data.object.args[0]
            if endness == "Iend_BE":
                arange = reversed(arange)

            assert memory.state.arch.byte_width == 8
            # TODO: Make UltraPage support architectures with greater byte_widths (but are still multiples of 8)
            for subaddr in arange:
                self.concrete_data[subaddr] = ival & 0xFF
                ival >>= 8
        else:
            # mark range as symbolic
            self.symbolic_bitmap[addr : addr + size] = b"\1" * size

            # set ending object
            try:
                endpiece = next(self.symbolic_data.irange(maximum=addr + size, reverse=True))
            except StopIteration:
                pass
            else:
                if endpiece != addr + size:
                    self.symbolic_data[addr + size] = self.symbolic_data[endpiece]

            # clear range
            for midpiece in self.symbolic_data.irange(maximum=addr + size - 1, minimum=addr, reverse=True):
                del self.symbolic_data[midpiece]

            # set.
            self.symbolic_data[addr] = data

    def merge(
        self,
        others: List["UltraPage"],
        merge_conditions,
        common_ancestor=None,
        page_addr: int = None,  # pylint: disable=arguments-differ
        memory=None,
        changed_offsets: Optional[Set[int]] = None,
    ):
        all_pages = [self] + others
        merged_to = None
        merged_objects = set()
        merged_offsets = set()

        if changed_offsets is None:
            changed_offsets = set()
            for other in others:
                changed_offsets |= self.changed_bytes(other, page_addr)

        for b in sorted(changed_offsets):
            if merged_to is not None and not b >= merged_to:
                l.info("merged_to = %d ... already merged byte 0x%x", merged_to, b)
                continue
            l.debug("... on byte 0x%x", b)

            memory_objects: List[Tuple[SimMemoryObject, Any]] = []
            concretes: List[Tuple[int, Any]] = []
            unconstrained_in: List[Tuple["UltraPage", Any]] = []
            our_mo: Optional[SimMemoryObject] = None

            # first get a list of all memory objects at that location, and
            # all memories that don't have those bytes
            for pg, fv in zip(all_pages, merge_conditions):
                if pg.symbolic_bitmap[b]:
                    mo = pg._get_object(b, page_addr)
                    if mo is not None:
                        l.info("... MO present in %s", fv)
                        memory_objects.append((mo, fv))
                        if pg is self:
                            our_mo = mo
                    else:
                        l.info("... not present in %s", fv)
                        unconstrained_in.append((pg, fv))
                else:
                    # concrete data
                    concretes.append((pg.concrete_data[b], fv))

            # fast path: no memory objects, no unconstrained positions, and only one concrete value
            if not memory_objects and not unconstrained_in and len({cv for cv, _ in concretes}) == 1:
                cv = concretes[0][0]
                self.store(b, cv, size=1, cooperate=True, page_addr=page_addr, memory=memory)
                continue

            # convert all concrete values into memory objects
            for cv, fv in concretes:
                mo = SimMemoryObject(claripy.BVV(cv, size=8), page_addr + b, "Iend_LE")
                memory_objects.append((mo, fv))

            mos = {mo for mo, _ in memory_objects}
            mo_bases = {mo.base for mo, _ in memory_objects}
            mo_lengths = {mo.length for mo, _ in memory_objects}

            if not unconstrained_in and not mos - merged_objects:
                continue

            # first, optimize the case where we are dealing with the same-sized memory objects
            if len(mo_bases) == 1 and len(mo_lengths) == 1 and not unconstrained_in:
                to_merge = [(mo.object, fv) for mo, fv in memory_objects]

                # Update `merged_to`
                merged_to = b + list(mo_lengths)[0]

                merged_val = self._merge_values(to_merge, memory_objects[0][0].length, memory=memory)
                if merged_val is None:
                    continue

                if our_mo is None:
                    # this object does not exist in the current page. do the store
                    new_object = SimMemoryObject(merged_val, page_addr + b, memory_objects[0][0].endness)
                    self.store(
                        b, new_object, size=list(mo_lengths)[0], cooperate=True, page_addr=page_addr, memory=memory
                    )
                    merged_objects.add(new_object)
                else:
                    # do the replacement
                    new_object = self._replace_memory_object(our_mo, merged_val, memory=memory)
                    merged_objects.add(new_object)

                merged_objects.update(mos)
                merged_offsets.add(b)

            else:
                # get the size that we can merge easily. This is the minimum of
                # the size of all memory objects and unallocated spaces.
                min_size = min(mo.length - (page_addr + b - mo.base) for mo, _ in memory_objects)
                for um, _ in unconstrained_in:
                    for i in range(0, min_size):
                        if um._contains(b + i, page_addr):
                            min_size = i
                            break
                merged_to = b + min_size
                l.info("... determined minimum size of %d", min_size)

                # Now, we have the minimum size. We'll extract/create expressions of that
                # size and merge them
                extracted = (
                    [(mo.bytes_at(page_addr + b, min_size), fv) for mo, fv in memory_objects] if min_size != 0 else []
                )
                created = [
                    (self._default_value(None, min_size, name=f"merge_uc_{uc.id}_{b:x}", memory=memory), fv)
                    for uc, fv in unconstrained_in
                ]
                to_merge = extracted + created

                merged_val = self._merge_values(to_merge, min_size, memory=memory)
                if merged_val is None:
                    continue

                self.store(
                    b,
                    merged_val,
                    size=len(merged_val) // memory.state.arch.byte_width,
                    inspect=False,
                    page_addr=page_addr,
                    memory=memory,
                )  # do not convert endianness again

                merged_offsets.add(b)

        return merged_offsets

    def concrete_load(self, addr, size, **kwargs):  # pylint: disable=arguments-differ
        if type(self.concrete_data) is bytearray:
            return (
                memoryview(self.concrete_data)[addr : addr + size],
                memoryview(self.symbolic_bitmap)[addr : addr + size],
            )
        else:
            return self.concrete_data[addr : addr + size], memoryview(self.symbolic_bitmap)[addr : addr + size]

    def changed_bytes(self, other, page_addr=None) -> Set[int]:
        changed_candidates = super().changed_bytes(other)
        if changed_candidates is None:
            changed_candidates = range(len(self.symbolic_bitmap))

        changes: Set[int] = set()

        for addr in changed_candidates:
            if self.symbolic_bitmap[addr] != other.symbolic_bitmap[addr]:
                changes.add(addr)
            elif self.symbolic_bitmap[addr] == 0:
                if self.concrete_data[addr] != other.concrete_data[addr]:
                    changes.add(addr)
            else:
                try:
                    aself = next(self.symbolic_data.irange(maximum=addr, reverse=True))
                except StopIteration:
                    aself = None
                try:
                    aother = next(other.symbolic_data.irange(maximum=addr, reverse=True))
                except StopIteration:
                    aother = None

                if aself is None and aother is None:
                    pass
                elif aself is None and aother is not None:
                    oobj = other.symbolic_data[aother]
                    if oobj.includes(addr + page_addr):
                        changes.add(addr)
                elif aother is None and aself is not None:
                    aobj = self.symbolic_data[aself]
                    if aobj.includes(addr + page_addr):
                        changes.add(addr)
                else:
                    real_addr = page_addr + addr
                    aobj = self.symbolic_data[aself]
                    oobj = other.symbolic_data[aother]

                    acont = aobj.includes(real_addr)
                    ocont = oobj.includes(real_addr)
                    if acont != ocont:
                        changes.add(addr)
                    elif acont is False:
                        pass
                    else:
                        abyte = aobj.bytes_at(real_addr, 1)
                        obyte = oobj.bytes_at(real_addr, 1)
                        if abyte is not obyte:
                            changes.add(addr)

        return changes

    def _contains(self, start: int, page_addr: int):
        if not self.symbolic_bitmap[start]:
            # concrete data
            return True
        else:
            # symbolic data or does not exist
            return self._get_object(start, page_addr) is not None

    def _get_object(self, start: int, page_addr: int, memory=None) -> Optional[SimMemoryObject]:
        try:
            place = next(self.symbolic_data.irange(maximum=start, reverse=True))
        except StopIteration:
            return None
        else:
            obj = self.symbolic_data[place]
            if obj.includes(start + page_addr):
                return obj
            elif memory is not None and obj.includes(start + page_addr + (1 << memory.state.arch.bits)):
                return obj
            else:
                return None

    def replace_all_with_offsets(self, offsets: Iterable[int], old: claripy.ast.BV, new: claripy.ast.BV, memory=None):
        memory_objects = set()
        for offset in sorted(list(offsets)):
            try:
                a = next(self.symbolic_data.irange(maximum=offset, reverse=True))
            except StopIteration:
                a = None

            if a is None:
                continue
            aobj = self.symbolic_data[a]
            memory_objects.add(aobj)

        replaced_objects_cache = {}
        for mo in memory_objects:
            replaced_object = None

            if mo.object in replaced_objects_cache:
                if mo.object is not replaced_objects_cache[mo.object]:
                    replaced_object = replaced_objects_cache[mo.object]

            else:
                replaced_object = mo.object.replace(old, new)
                replaced_objects_cache[mo.object] = replaced_object
                if mo.object is replaced_object:
                    # The replace does not really occur
                    replaced_object = None

            if replaced_object is not None:
                self._replace_memory_object(mo, replaced_object, memory=memory)

    def _replace_memory_object(self, old: SimMemoryObject, new_content: claripy.Bits, memory=None):
        """
        Replaces the memory object `old` with a new memory object containing `new_content`.

        :param old:         A SimMemoryObject (i.e., one from :func:`memory_objects_for_hash()` or :func:`
                            memory_objects_for_name()`).
        :param new_content: The content (claripy expression) for the new memory object.
        :returns: the new memory object
        """

        if (
            old.object.size() if not old.is_bytes else len(old.object) * self.state.arch.byte_width
        ) != new_content.size():
            raise SimMemoryError("memory objects can only be replaced by the same length content")

        new = SimMemoryObject(new_content, old.base, old.endness, byte_width=old._byte_width)
        for k in list(self.symbolic_data):
            if self.symbolic_data[k] is old:
                self.symbolic_data[k] = new

        if isinstance(new.object, claripy.ast.BV):  # pylint:disable=isinstance-second-argument-not-valid-type
            for b in range(old.base, old.base + old.length):
                self._update_mappings(b, old.object, new.object, memory=memory)

        return new



#!/usr/bin/env python3
# pylint:disable=missing-class-docstring,no-self-use
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestTypehoon(unittest.TestCase):
    def test_smoketest(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "linked_list"), auto_load_libs=False)
        cfg = p.analyses.CFG(data_references=True, normalize=True)

        main_func = cfg.kb.functions["sum"]

        vr = p.analyses.VariableRecoveryFast(main_func)
        p.analyses.CompleteCallingConventions()

        # import pprint
        tcons = vr.type_constraints
        # pprint.pprint(vr._outstates[0x4005b2].typevars._typevars)
        # pprint.pprint(tcons)

        _ = p.analyses.Typehoon(tcons)
        # pprint.pprint(t.simtypes_solution)

        # convert function blocks to AIL blocks
        # clinic = p.analyses.Clinic(main_func)

        # t = p.analyses.Typehoon(main_func) #, clinic)
        # print(t)

    def test_type_inference_byte_pointer_cast(self):
        proj = angr.Project(os.path.join(test_location, "i386", "type_inference_1"), auto_load_libs=False)
        cfg = proj.analyses.CFG(data_references=True, normalize=True)
        main_func = cfg.kb.functions["main"]
        proj.analyses.VariableRecoveryFast(main_func)
        proj.analyses.CompleteCallingConventions()

        dec = proj.analyses.Decompiler(main_func)
        assert "->field_0 = 10;" in dec.codegen.text
        assert "->field_4 = 20;" in dec.codegen.text
        assert "->field_8 = 808464432;" in dec.codegen.text
        assert "->field_c = 0;" in dec.codegen.text


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring
from unittest import main, TestCase

from archinfo import ArchMIPS32

from angr.calling_conventions import SimRegArg
from angr.knowledge_plugins.key_definitions.atoms import Atom, Register


class TestAtoms(TestCase):
    def test_from_argument_instanciate_a_Register_when_given_a_SimRegArg(self):
        argument = SimRegArg("r0", 4)
        arch = ArchMIPS32()

        result = Atom.from_argument(argument, arch)

        self.assertTrue(isinstance(result, Register))
        self.assertEqual(result.reg_offset, arch.registers["r0"][0])
        self.assertEqual(result.size, 4)


if __name__ == "__main__":
    main()



"""Defines analysis that will generate a dynamic data-dependency graph"""
import logging
import math
from typing import Optional, List, Union, Dict, Set, Tuple, TYPE_CHECKING

from networkx import DiGraph

import claripy
from claripy.ast.bv import BV
from .dep_nodes import DepNodeTypes, ConstantDepNode, MemDepNode, VarDepNode, RegDepNode, TmpDepNode
from .sim_act_location import SimActLocation, DEFAULT_LOCATION, ParsedInstruction
from .. import Analysis
from ...analyses import AnalysesHub
from ...errors import AngrDDGError, AngrAnalysisError, SimValueError
from ...state_plugins import SimActionData
from ...storage import DefaultMemory

if TYPE_CHECKING:
    from .dep_nodes import BaseDepNode
    from angr import SimState

logger = logging.getLogger(name=__name__)


class NodalAnnotation(claripy.Annotation):
    """
    Allows a node to be stored as an annotation to a BV in a DefaultMemory instance
    """

    def __init__(self, node: "BaseDepNode"):
        self.node = node

    @property
    def relocatable(self) -> bool:
        """Can not be relocated in a simplification"""
        return False

    @property
    def eliminatable(self):
        """Can not be eliminated in a simplification"""
        return False


class DataDependencyGraphAnalysis(Analysis):
    """
    This is a DYNAMIC data dependency graph that utilizes a given SimState to produce a DDG graph that is accurate to
    the path the program took during execution.

    This analysis utilizes the SimActionData objects present in the provided SimState's action history to generate the
    dependency graph.
    """

    def __init__(
        self,
        end_state: "SimState",
        start_from: Optional[int] = None,
        end_at: Optional[int] = None,
        block_addrs: Optional[List[int]] = None,
    ):
        """
        :param end_state: Simulation state used to extract all SimActionData
        :param start_from: An address or None, Specifies where to start generation of DDG
        :param end_at: An address or None, Specifies where to end generation of DDG
        :param iterable or None block_addrs: List of block addresses that the DDG analysis should be run on
        """
        self._graph: Optional[DiGraph] = None
        self._simplified_graph: Optional[DiGraph] = None
        self._sub_graph: Optional[DiGraph] = None

        self._end_state = end_state
        self._start_from = start_from if start_from else self.project.entry
        self._end_at = end_at
        self._block_addrs = frozenset(block_addrs) if block_addrs else frozenset()

        self._register_file = DefaultMemory(memory_id="reg")  # Tracks the current state of all parsed registers
        self._register_file.set_state(self._end_state)
        self._memory_map = DefaultMemory(memory_id="mem")  # Tracks the current state of all parsed memory addresses
        self._memory_map.set_state(self._end_state)
        self._tmp_nodes: Dict[str, TmpDepNode] = {}  # Per-block: Maps temp name to its current node
        self._constant_nodes: Dict[int, ConstantDepNode] = {}  # Per program: Maps values to their ConstantDepNodes
        self._actions: List["SimActionData"] = []

        # Used by parser to track instruction addresses processed
        self._parsed_ins_addrs: List[ParsedInstruction] = []  # Instruction address, min stmt_idx, max stmt_idx

        # Parameter sanity check
        if self._block_addrs and self._end_at:
            raise AngrDDGError("Can not specify BOTH start/end addresses and a block list!")

        self._work()

    @property
    def graph(self) -> Optional[DiGraph]:
        return self._graph

    @property
    def simplified_graph(self) -> Optional[DiGraph]:
        return self._simplified_graph

    @property
    def sub_graph(self) -> Optional[DiGraph]:
        return self._sub_graph

    def _pop(self) -> Optional["SimActionData"]:
        """
        Safely pops the first action, if it exists.
        """
        return self._actions.pop(0) if self._actions else None

    def _peek(self, idx: int = 0) -> Optional["SimActionData"]:
        """
        Safely returns the first action, if it exists, without removing it

        :param idx: Index to peek at, default 0
        """
        return self._actions[idx] if len(self._actions) > idx else None

    def _peek_type(self) -> str:
        """
        Safely returns the type of the first action, if it exists, otherwise ''
        """
        return self._actions[0].type if self._actions else ""

    def _peek_action(self) -> str:
        """
        Safely returns the type of the first action, if it exists, otherwise ''
        """
        return self._actions[0].action if self._actions else ""

    def _set_active_node(self, node: "BaseDepNode"):
        arch_bw = self._end_state.arch.bits
        if isinstance(node, RegDepNode):
            annotated_bv = node.ast.annotate(NodalAnnotation(node))
            annotated_bv = annotated_bv.zero_extend(arch_bw - annotated_bv.size())
            self._register_file.store(node.reg, annotated_bv)
        elif isinstance(node, MemDepNode):
            annotated_bv = node.ast.annotate(NodalAnnotation(node))
            self._memory_map.store(node.addr, annotated_bv)
        elif isinstance(node, TmpDepNode):
            self._tmp_nodes[node.arch_name] = node
        elif isinstance(node, ConstantDepNode):
            self._constant_nodes[node.value] = node
        else:
            raise TypeError(f"{str(node)} is node a DepNode")

    def _get_active_node(self, node: "BaseDepNode") -> Optional["BaseDepNode"]:
        """
        Retrieves the currently active node from the provided node type's storage data structure
        :param node: Node to retrieve the ancestor of
        :return: The ancestor, if it exists
        """
        ret_node = None
        if isinstance(node, RegDepNode):
            reg_data = self._register_file.load(node.reg, self._end_state.arch.byte_width)
            if not reg_data.symbolic and len(reg_data.annotations) > 0:
                ret_node = reg_data.annotations[0].node
        elif isinstance(node, MemDepNode):
            mem_data = self._memory_map.load(node.addr, node.width)
            if not mem_data.symbolic and len(mem_data.annotations) > 0:
                ret_node = mem_data.annotations[0].node
        elif isinstance(node, TmpDepNode):
            ret_node = self._tmp_nodes.get(node.arch_name, None)
        elif isinstance(node, ConstantDepNode):
            ret_node = self._constant_nodes.get(node.value, None)
        else:
            raise TypeError(f"{str(node)} is node a DepNode")
        return ret_node

    def _get_or_create_graph_node(
        self,
        type_: int,
        sim_act: "SimActionData",
        val: Tuple["BV", int],
        *constructor_params,
    ) -> "BaseDepNode":
        """
        If the node already exists in the graph, that node is returned. Otherwise, a new node is created
        :param _type: Type of node to check/create
        :param sim_act: The SimActionData associated with the node
        :param val: The value to be assigned to the node, represented as a Tuple containing its BV and evaluation
        :param constructor_params: Variadic list of arguments to supply for node lookup / creation
        :return: A reference to a node with the given parameters
        """

        # Always create a new write node

        if type_ is DepNodeTypes.Register or type_ is DepNodeTypes.Tmp:
            # Create and configure new node
            node_cls = RegDepNode if type_ is DepNodeTypes.Register else TmpDepNode
            ret_node = node_cls(sim_act, *constructor_params)
            ret_node.ast = val[0]
            ret_node.value = val[1]
            if not ret_node.arch_name:
                ret_node.arch_name = sim_act.storage

            self._graph.add_node(ret_node)

        elif type_ is DepNodeTypes.Memory:
            ret_node = MemDepNode(sim_act, *constructor_params)
            ret_node.ast = val[0]
            ret_node.value = val[1]
            self._graph.add_node(ret_node)
        elif type_ is DepNodeTypes.Constant:
            val_int = val[1]
            if val_int not in self._constant_nodes:
                self._constant_nodes[val_int] = ConstantDepNode(sim_act, val_int)
                self._graph.add_node(self._constant_nodes[val_int])
            ret_node = self._constant_nodes[val_int]
        else:
            raise TypeError("Type must be a type of DepNode.")

        return ret_node

    def _get_dep_node(
        self, dep_type: int, sim_act: SimActionData, var_src: Union[int, "BV"], val: Union[int, BV]
    ) -> "BaseDepNode":
        if isinstance(var_src, BV):
            var_src = self._end_state.solver.eval(var_src)

        val_ast = val
        if isinstance(val, BV):
            val = self._end_state.solver.eval(val)

        var_node = self._get_or_create_graph_node(dep_type, sim_act, (val_ast, val), *[var_src])

        return var_node

    def _get_generic_node(self, action: SimActionData) -> "BaseDepNode":
        def node_attributes(act: SimActionData) -> tuple:
            ac = act.action
            ty = act.type

            if ty not in [SimActionData.REG, SimActionData.TMP, SimActionData.MEM]:
                raise AngrAnalysisError(f"Unsupported type: <{ty}>!")
            if ac not in [SimActionData.WRITE, SimActionData.READ]:
                raise AngrAnalysisError(f"Unsupported action type: <{ac}>!")

            if ty == SimActionData.REG:
                if ac == SimActionData.READ:
                    tup = DepNodeTypes.Register, act.all_objects[0].ast, act.data.ast
                else:
                    tup = DepNodeTypes.Register, act.all_objects[0].ast, act.actual_value.ast
            elif ty == SimActionData.TMP:
                tup = DepNodeTypes.Tmp, act.tmp, act.all_objects[1].ast
            else:
                # Must be of type MEM, as we have already done a check
                tup = DepNodeTypes.Memory, act.addr.ast, act.data.ast
            return tup

        dep_type, var_src, val = node_attributes(action)
        return self._get_dep_node(dep_type, action, var_src, val)

    def _node_value_cmp(self, n1: "BaseDepNode", n2: "BaseDepNode") -> bool:
        """
        Performs a BV based comparison on the values of the nodes
        :param n1: First node to compare
        :param n2: Second node to compare
        :return: Whether the values of the nodes, bit-width dependent, are equivalent
        """

        # Zero-extend both values to the nearest byte for comparison
        def zero_extend_to_next_byte(ast: BV) -> BV:
            goal_bw = 8 * (math.ceil(ast.size() / 8))
            return ast.zero_extend(goal_bw - ast.size())

        ext_n1 = zero_extend_to_next_byte(n1.ast)
        ext_n2 = zero_extend_to_next_byte(n2.ast)

        if ext_n1.size() % 8 != 0 or ext_n2.size() % 8 != 0:
            # Something went wrong in extension
            raise ValueError("Cannot compare BV that aren't of a bit-width that is a multiple of 8!")

        cmp_bw = min(ext_n1.size(), ext_n2.size())
        cmp_byte_width = cmp_bw // 8

        n1_idx = ext_n1.size() // 8 - cmp_byte_width
        n2_idx = ext_n2.size() // 8 - cmp_byte_width
        n1_cmp_ast = ext_n1.get_bytes(n1_idx, cmp_byte_width)
        n2_cmp_ast = ext_n2.get_bytes(n2_idx, cmp_byte_width)
        try:
            return self._end_state.solver.eval_one(n1_cmp_ast == n2_cmp_ast)
        except SimValueError as s_val_err:
            logger.error(s_val_err)
            return False

    def _parse_action(self) -> "BaseDepNode":
        return self._get_generic_node(self._pop())

    def _parse_read_statement(self, read_nodes: Optional[Dict[int, List["BaseDepNode"]]] = None) -> "BaseDepNode":
        act = self._peek()
        read_node = self._parse_action()
        ancestor_node = self._get_active_node(read_node)

        if ancestor_node:
            # An ancestor exists for this read, and must be linked
            self._graph.add_edge(ancestor_node, read_node, label="ancestor")

        # Determine if read node should be marked a dependency of a constant value
        has_ancestor_and_new_value = ancestor_node is not None and not self._node_value_cmp(ancestor_node, read_node)
        is_orphan_and_new_value = ancestor_node is None and not self._constant_nodes.get(read_node.value)

        if has_ancestor_and_new_value or is_orphan_and_new_value:
            # This is a value that isn't inherited from a previous statement
            val_node = self._get_or_create_graph_node(DepNodeTypes.Constant, act, read_node.value_tuple(), True)
            self._graph.add_edge(val_node, read_node)

        read_nodes.setdefault(read_node.value, [])
        read_nodes[read_node.value].append(read_node)

        return read_node

    def _create_dep_edges(self, act, write_node, read_nodes: Dict[int, List["BaseDepNode"]]) -> bool:
        """Last resort for linking dependencies"""
        # Check tmp and reg deps
        var_read_nodes = []
        for nodes in read_nodes.values():
            for node in nodes:
                if isinstance(node, VarDepNode):
                    var_read_nodes.append(node)

        possible_dep_nodes = {node.reg: node for node in var_read_nodes}

        dep_found = False

        for tmp_off in act.tmp_deps:
            dep_node = possible_dep_nodes.get(tmp_off, None)
            if dep_node and isinstance(dep_node, TmpDepNode):
                dep_found = True
                self._graph.add_edge(dep_node, write_node, label="unknown_dep")

        for reg_off in act.reg_deps:
            dep_node = possible_dep_nodes.get(reg_off, None)
            if dep_node and not isinstance(dep_node, TmpDepNode):
                dep_found = True
                self._graph.add_edge(dep_node, write_node, label="unknown_dep")

        return dep_found

    def _parse_var_statement(self, read_nodes: Optional[Dict[int, List["BaseDepNode"]]] = None) -> SimActLocation:
        act = self._peek()
        act_loc = SimActLocation(act.bbl_addr, act.ins_addr, act.stmt_idx)

        if act.action == SimActionData.WRITE:
            write_node = self._parse_action()

            src_nodes = read_nodes.get(write_node.value, None)
            if src_nodes:
                # Write value came from a previous read value
                for src_node in src_nodes:
                    self._graph.add_edge(src_node, write_node, label="val")
                read_nodes.pop(write_node.value, None)  # Remove from read nodes before backup edge finder

                # Helps with edge cases, ensures no more dependencies remain as tracked in tmp_deps and reg_deps
                # per the SAO
                self._create_dep_edges(act, write_node, read_nodes)
            elif len(read_nodes) == 0:
                # No reads in this instruction before first write, so its value is direct
                # if ConstantDepNode(write_node.value) not in self._canonical_graph_nodes:
                val_node = self._get_or_create_graph_node(DepNodeTypes.Constant, act, write_node.value_tuple(), True)
                self._graph.add_edge(val_node, write_node)
            elif len(read_nodes) == 1:
                # Some calculation must have been performed on the value of the single read
                stmt_read_nodes = list(read_nodes.values())[0]

                for stmt_read_node in stmt_read_nodes:
                    # diff = list(read_nodes.keys())[0] - write_node.value
                    # edge_label = f"{'-' if diff > 0 else '+'} {abs(diff)}"
                    self._graph.add_edge(stmt_read_node, write_node)  # label=edge_label)
            else:
                dep_found = self._create_dep_edges(act, write_node, read_nodes)
                if not dep_found:
                    logger.warning("Node <%r> written to without tracked value source!", write_node)

            self._set_active_node(write_node)
            return act_loc
        else:
            read_node = self._parse_read_statement(read_nodes=read_nodes)
            self._set_active_node(read_node)

            # Sometimes an R is the last action in a statement
            return (
                self._parse_statement(read_nodes) if self._peek() and act.stmt_idx == self._peek().stmt_idx else act_loc
            )

    def _parse_mem_statement(self, read_nodes: Optional[Dict[int, List["BaseDepNode"]]] = None) -> SimActLocation:
        act = self._peek()
        act_loc = SimActLocation(act.bbl_addr, act.ins_addr, act.stmt_idx)

        if act.action == SimActionData.WRITE:
            mem_node = MemDepNode.cast_to_mem(self._parse_action())
            src_nodes = read_nodes.get(mem_node.value, None)
            if src_nodes:
                # Value being written to, address came from previous read
                for src_node in src_nodes:
                    self._graph.add_edge(src_node, mem_node, label="val")
            elif len(read_nodes) == 1 and read_nodes.get(mem_node.addr, None):
                # Only read thus far was for the memory address, value is direct
                val_node = self._get_or_create_graph_node(DepNodeTypes.Constant, act, mem_node.value_tuple(), True)
                self._graph.add_edge(val_node, mem_node)
            else:
                raise AngrAnalysisError(f"Unexpected MemWrite pattern encountered! <{act}>")
            self._set_active_node(mem_node)
            ret_val = act_loc
        else:
            mem_node = self._parse_read_statement(read_nodes)
            self._set_active_node(mem_node)
            # Sometimes an R is the last action in a statement
            ret_val = None if self._peek() and act.stmt_idx == self._peek().stmt_idx else act_loc

        # Handle the address of the mem R/W
        addr_source_nodes = read_nodes.get(mem_node.addr, None)
        if addr_source_nodes:
            for addr_source_node in addr_source_nodes:
                self._graph.add_edge(addr_source_node, mem_node, label="addr_source")

        return ret_val if ret_val else self._parse_statement(read_nodes)

    def _parse_statement(self, read_nodes: Optional[Dict[int, List["BaseDepNode"]]] = None) -> SimActLocation:
        """
        statement -> write_var | write_mem
        statement -> read_var | write_mem statement
        :return: The instruction address associated with the statement
        """
        read_nodes = read_nodes if read_nodes else {}
        sim_act = self._peek()
        nxt_act = self._peek(1)
        if not sim_act:
            return DEFAULT_LOCATION

        # Some sanity checks
        if sim_act.action not in [SimActionData.WRITE, SimActionData.READ]:
            raise AngrAnalysisError(f"Statement with unsupported action encountered: <{sim_act.action}>")
        if sim_act.type not in [SimActionData.TMP, SimActionData.MEM, SimActionData.REG]:
            raise AngrAnalysisError(f"Statement with unsupported type encountered: <{sim_act.type}>")
        if (
            sim_act.action == SimActionData.WRITE
            and nxt_act
            and nxt_act.ins_addr == sim_act.ins_addr
            and nxt_act.stmt_idx == sim_act.stmt_idx
        ):
            raise AngrAnalysisError(
                "Statement must end with a write," f"but {self._peek(1)} follows a write!", self._peek(1)
            )

        if sim_act.type == SimActionData.MEM:
            return self._parse_mem_statement(read_nodes)
        else:
            return self._parse_var_statement(read_nodes)  # Tmp or Reg

    def _parse_instruction(self) -> SimActLocation:
        """
        Grammar:
        instruction -> statement
        instruction -> statement instruction

        :returns: The instruction address and last statement index of the parsed instruction
        """

        while True:
            loc = self._parse_statement()

            if not self._actions or loc.ins_addr != self._peek().ins_addr:
                # End of instruction
                return loc

    def _parse_block(self):
        """
        block -> instruction
        block -> instruction block
        """

        while True:
            start_stmt_idx = self._peek().stmt_idx  # Statement index of first statement in instruction

            end_loc = self._parse_instruction()

            # Add most recently parsed instruction to instructions data structure
            parsed_ins = ParsedInstruction(end_loc.ins_addr, start_stmt_idx, end_loc.stmt_idx)
            self._parsed_ins_addrs.insert(0, parsed_ins)

            if not self._actions or end_loc.bbl_addr != self._peek().bbl_addr:
                # Block continues with at least one more instruction
                break

    def _parse_blocks(self):
        """
        blocks -> block
        blocks -> block blocks
        """
        if self._actions:
            self._tmp_nodes.clear()  # Nodes are unique to a block
            self._parse_block()
            self._parse_blocks()

    def _filter_sim_actions(self) -> List[SimActionData]:
        """
        Using the user's start/end address OR block address list parameters,
        filters the actions down to those that are relevant
        :return: The relevant actions
        """

        if self._block_addrs:
            # Retrieve all actions from the given block(s)
            relevant_actions = list(
                filter(lambda act: act.bbl_addr in self._block_addrs, list(self._end_state.history.actions.hardcopy))
            )
        elif self._end_at:
            relevant_actions = self._end_state.history.filter_actions(
                start_block_addr=self._start_from, end_block_addr=self._end_at
            )[::-1]
        else:
            relevant_actions = self._end_state.history.filter_actions(start_block_addr=self._start_from)[::-1]

        # We only care about SimActionData objects for this analysis
        relevant_actions = list(
            filter(lambda act: isinstance(act, SimActionData) and act.sim_procedure is None, relevant_actions)
        )
        return relevant_actions

    def _work(self):
        """
        Generates the DDG
        """
        self._graph = DiGraph()
        self._actions = self._filter_sim_actions()

        # ins_str = curr_node
        # ins_addr = self._actions[0].ins_addr
        # stmt_addr = self._actions[0].stmt_idx
        #
        # for act in self._actions:
        #     if act.ins_addr != ins_addr:
        #         ins_addr = act.ins_addr
        #         print(ins_str)
        #         ins_str = f'{hex(ins_addr)}: '
        #     if stmt_addr != act.stmt_idx:
        #         stmt_addr = act.stmt_idx
        #         ins_str += ' '
        #
        #     ins_str += 'R' if act.action is SimActionData.READ else 'W'
        self._parse_blocks()

        # Create a simplified version of the graph
        self._simplified_graph = self._simplify_graph(self._graph)

    @staticmethod
    def _simplify_graph(G: DiGraph) -> DiGraph:
        """
        Performs an in-place removal of all tmp nodes and reconnects var nodes and mem nodes.
        :param G: Graph to be simplified
        """

        g0 = G.copy()
        tmp_nodes = [n for n in g0.nodes() if isinstance(n, TmpDepNode)]
        for curr_node in tmp_nodes:
            # Node must be removed and predecessor(s) connected to successor(s)
            in_edges = list(g0.in_edges(curr_node, data=True))
            out_edges = list(g0.edges(curr_node, data=True))
            for pred, _, _ in in_edges:
                g0.remove_edge(pred, curr_node)

            for _, suc, _ in out_edges:
                g0.remove_edge(curr_node, suc)

            for pred, _, data_in in in_edges:
                for _, suc, data_out in out_edges:
                    data = data_in.copy()
                    data.update(data_out)
                    g0.add_edge(pred, suc, **data)

            g0.remove_node(curr_node)

        return g0

    @staticmethod
    def _get_related_nodes(G: DiGraph, curr_node: "BaseDepNode", nodes: Set["BaseDepNode"], get_ancestors: bool):
        nodes.add(curr_node)

        next_nodes = G.predecessors(curr_node) if get_ancestors else G.successors(curr_node)

        if next_nodes:
            for p in next_nodes:
                if not isinstance(p, RegDepNode) or p.arch_name not in ["rsp", "rbp"]:
                    # Tracking RSP/RBP as sources just clutters the graph...
                    DataDependencyGraphAnalysis._get_related_nodes(G, p, nodes, get_ancestors)
        else:
            return

    def get_data_dep(self, g_node: "BaseDepNode", include_tmp_nodes: bool, backwards: bool) -> Optional[DiGraph]:
        # We have a matching node and can proceed to build a subgraph
        if g_node in self._graph:
            relevant_nodes = set()
            g = self._graph if include_tmp_nodes else self._simplified_graph

            DataDependencyGraphAnalysis._get_related_nodes(g, g_node, relevant_nodes, backwards)
            self._sub_graph = g.subgraph(relevant_nodes).copy()
            return self._sub_graph
        else:
            logger.error("No node %r in existing graph.", g_node)
            return None


# register this analysis
AnalysesHub.register_default("DataDep", DataDependencyGraphAnalysis)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.storage"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestPermissions(unittest.TestCase):
    def test_nx(self):
        nx_amd64 = angr.Project(os.path.join(test_location, "x86_64", "memmove"), auto_load_libs=False)
        es = nx_amd64.factory.entry_state()

        # .text should be PROT_READ|PROT_EXEC
        assert es.solver.eval(es.memory.permissions(nx_amd64.entry)) == 5

        # load stack to initialize page
        es.memory.load(es.regs.sp, 4)

        # stack should be PROT_READ|PROT_WRITE
        assert es.solver.eval(es.memory.permissions(es.regs.sp)) == 3


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.exploration_techniques"  # pylint:disable=redefined-builtin

import os
import gc
import unittest

import claripy

from angr.exploration_techniques.spiller import Spiller
import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


def _bin(*s):
    return os.path.join(test_location, *s)


def pickle_callback(state):
    state.globals["pickled"] = True


def unpickle_callback(sid, state):  # pylint:disable=unused-argument
    state.globals["unpickled"] = True


def priority_key(state):
    return state.addr * state.history.depth  # to help ensure determinism


class TestSpiller(unittest.TestCase):
    @classmethod
    def setUp(self):
        # clean up AST cache in claripy, because a cached AST might believe it
        # has been stored in ana after we clean up the ana storage

        claripy.ast.bv._bvv_cache.clear()
        claripy.ast.bv.BV._hash_cache.clear()

    def test_basic(self):
        project = angr.Project(_bin("cgc", "sc2_0b32aa01_01"), auto_load_libs=False)
        state = project.factory.entry_state()
        spiller = Spiller(pickle_callback=pickle_callback, unpickle_callback=unpickle_callback)
        spiller._pickle([state])

        del state
        gc.collect()
        state = spiller._unpickle(1)[0]

        assert state.globals["pickled"]
        assert state.globals["unpickled"]

    def test_palindrome2(self):
        project = angr.Project(_bin("cgc", "sc2_0b32aa01_01"), auto_load_libs=False)
        pg = project.factory.simulation_manager()
        limiter = angr.exploration_techniques.LengthLimiter(max_length=250)
        pg.use_technique(limiter)

        spiller = Spiller(
            pickle_callback=pickle_callback, unpickle_callback=unpickle_callback, priority_key=priority_key
        )
        pg.use_technique(spiller)
        # pg.step(until=lambda lpg: len(lpg.active) == 10)
        # pg.step(until=lambda lpg: len(lpg.spill_stage) > 15)
        # pg.step(until=lambda lpg: spiller._pickled_paths)
        pg.run()

        assert spiller._ever_pickled > 0
        assert spiller._ever_unpickled == spiller._ever_pickled
        assert all(
            ("pickled" not in state.globals and "unpickled" not in state.globals)
            or (state.globals["pickled"] and state.globals["unpickled"])
            for state in pg.cut
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long

import unittest

import angr


class TestUnlink(unittest.TestCase):
    def test_file_unlink(self):
        # Initialize a blank state with an arbitrary errno location
        state = angr.SimState(arch="AMD64", mode="symbolic")
        state.libc.errno_location = 0xA0000000
        state.libc.errno = 0

        # Create a file 'test'
        fd = state.posix.open(b"test", 1)
        state.posix.close(fd)

        # Ensure 'test' was in fact created
        assert b"/test" in state.fs._files

        # Store the filename in memory
        path_addr = 0xB0000000
        state.memory.store(path_addr, b"test\x00")

        # Unlink 'test': should return 0 and leave ERRNO unchanged
        unlink = angr.SIM_PROCEDURES["posix"]["unlink"]()
        state.scratch.sim_procedure = unlink
        rval = unlink.execute(state, arguments=[path_addr]).ret_expr
        assert rval == 0
        assert state.solver.eval(state.libc.errno) == 0

        # Check that 'test' was in fact deleted
        assert state.fs._files == {}

        # Unlink again: should return -1 and set ERRNO to ENOENT
        unlink = angr.SIM_PROCEDURES["posix"]["unlink"]()
        state.scratch.sim_procedure = unlink
        rval = unlink.execute(state, arguments=[path_addr]).ret_expr
        assert rval == -1
        assert state.solver.eval(state.libc.errno) == state.posix.ENOENT


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
import unittest

from angr.state_plugins.callstack import CallStack


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestCallstack(unittest.TestCase):
    def test_empty_stack(self):
        cs = CallStack()

        # Initial setting: just assume the control flow starts from 0x300000
        cs = cs.call(None, 0x300000, None, 0xFFFFFFF0)

        # Calling 0x401000 from 0x400000. When it returns, it should return to 0x400004.
        # The stack pointer after entering the new function should be 0xffffff00
        # Note: this means on platforms like x86 and AMD64 where CALL instruction actually pushes the ret address on to
        # the stack and modifies the stack pointer, the user should adjust stack pointer accordingly (minus 4 or 8, for
        # example) before passing to CallStack.call(). CallStack has no way to know what the architecture it is used on.
        cs = cs.call(0x400000, 0x401000, 0x400004, 0xFFFFFF00)

        cs = cs.call(0x401008, 0x402000, 0x40100C, 0xFFFFFE80)

        assert cs.current_function_address == 0x402000
        assert cs.current_stack_pointer == 0xFFFFFE80

        # Return to 0x40100c
        cs = cs.ret(0x40100C)

        assert cs.current_function_address == 0x401000
        assert cs.current_stack_pointer == 0xFFFFFF00

        cs = cs.ret(0x400004)

        assert cs.current_function_address == 0x300000
        assert cs.current_stack_pointer == 0xFFFFFFF0

        # We return one more time to see what happens
        # Ideally nothing should be popped out
        cs = cs.ret(0x200000)

        assert cs.current_function_address == 0x300000
        assert cs.current_stack_pointer == 0xFFFFFFF0

        # Final return!
        cs = cs.ret(None)

        assert cs.current_function_address == 0
        assert cs.current_stack_pointer == 0


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
from unittest import main, TestCase

from angr.knowledge_plugins.cfg import CFGManager


class TestCFGManager(TestCase):
    def setUp(self):
        self.cfg_manager = CFGManager(None)

    def test_when_both_cfg_emulated_and_cfg_fast_are_present(self):
        self.cfg_manager["CFGEmulated"] = "fake CFGEmulated"
        self.cfg_manager["CFGFast"] = "fake CFGFast"

        result = self.cfg_manager.get_most_accurate()
        self.assertEqual(result, "fake CFGEmulated")

    def test_when_only_cfg_emulated_is_present(self):
        self.cfg_manager["CFGEmulated"] = "fake CFGEmulated"

        result = self.cfg_manager.get_most_accurate()
        self.assertEqual(result, "fake CFGEmulated")

    def test_when_only_cfg_fast_is_present(self):
        self.cfg_manager["CFGFast"] = "fake CFGFast"

        result = self.cfg_manager.get_most_accurate()
        self.assertEqual(result, "fake CFGFast")

    def test_when_no_cfg_is_present(self):
        result = self.cfg_manager.get_most_accurate()
        self.assertEqual(result, None)


if __name__ == "__main__":
    main()



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("sfc.dll")
prototypes = \
    {
        #
        'SfcGetNextProtectedFile': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"FileName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "FileNumber": SimTypeInt(signed=False, label="UInt32")}, name="PROTECTED_FILE_DATA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RpcHandle", "ProtFileData"]),
        #
        'SfcIsFileProtected': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["RpcHandle", "ProtFileName"]),
        #
        'SfcIsKeyProtected': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["KeyHandle", "SubKeyName", "KeySam"]),
        #
        'SfpVerifyFile': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pszFileName", "pszError", "dwErrSize"]),
        #
        'SRSetRestorePointA': SimTypeFunction([SimTypePointer(SimStruct({"dwEventType": SimTypeInt(signed=False, label="RESTOREPOINTINFO_EVENT_TYPE"), "dwRestorePtType": SimTypeInt(signed=False, label="RESTOREPOINTINFO_TYPE"), "llSequenceNumber": SimTypeLongLong(signed=True, label="Int64"), "szDescription": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 64)}, name="RESTOREPOINTINFOA", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"nStatus": SimTypeInt(signed=False, label="UInt32"), "llSequenceNumber": SimTypeLongLong(signed=True, label="Int64")}, name="STATEMGRSTATUS", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pRestorePtSpec", "pSMgrStatus"]),
        #
        'SRSetRestorePointW': SimTypeFunction([SimTypePointer(SimStruct({"dwEventType": SimTypeInt(signed=False, label="RESTOREPOINTINFO_EVENT_TYPE"), "dwRestorePtType": SimTypeInt(signed=False, label="RESTOREPOINTINFO_TYPE"), "llSequenceNumber": SimTypeLongLong(signed=True, label="Int64"), "szDescription": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 256)}, name="RESTOREPOINTINFOW", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"nStatus": SimTypeInt(signed=False, label="UInt32"), "llSequenceNumber": SimTypeLongLong(signed=True, label="Int64")}, name="STATEMGRSTATUS", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pRestorePtSpec", "pSMgrStatus"]),
    }

lib.set_prototypes(prototypes)



import angr


class openlog(angr.SimProcedure):
    # pylint:disable=arguments-differ

    def run(self, ident, option, facility):
        # A stub for openlog that does not do anything yet.
        return



# pylint:disable=unused-argument
from typing import Optional, Dict, Set, Tuple, Generator, TYPE_CHECKING
from enum import Enum

import networkx  # pylint:disable=unused-import

import ailment


if TYPE_CHECKING:
    from angr.knowledge_plugins.functions import Function


class MultipleBlocksException(Exception):
    """
    An exception that is raised in _get_block() where multiple blocks satisfy the criteria but only one block was
    requested.
    """

    pass


class OptimizationPassStage(Enum):
    """
    Enums about optimization pass stages.

    Note that the region identification pass (RegionIdentifier) may modify existing AIL blocks *without updating the
    topology of the original AIL graph*. For example, loop successor refinement may modify create a new AIL block with
    an artificial address, and alter existing jump targets of jump statements and conditional jump statements to point
    to this new block. However, loop successor refinement does not update the topology of the original AIL graph, which
    means this new AIL block does not exist in the original AIL graph. As a result, until this behavior of
    RegionIdentifier changes in the future, DURING_REGION_IDENTIFICATION optimization passes should not modify existing
    jump targets.
    """

    AFTER_AIL_GRAPH_CREATION = 0
    AFTER_SINGLE_BLOCK_SIMPLIFICATION = 1
    AFTER_GLOBAL_SIMPLIFICATION = 2
    AFTER_VARIABLE_RECOVERY = 3
    BEFORE_REGION_IDENTIFICATION = 4
    DURING_REGION_IDENTIFICATION = 5
    AFTER_STRUCTURING = 6


class BaseOptimizationPass:
    """
    The base class for any optimization pass.
    """

    ARCHES = []  # strings of supported architectures
    PLATFORMS = []  # strings of supported platforms. Can be one of the following: "win32", "linux"
    STAGE: int = None  # Specifies when this optimization pass should be executed
    STRUCTURING: Optional[
        str
    ] = None  # specifies if this optimization pass is specific to a certain structuring algorithm

    def __init__(self, func):
        self._func: "Function" = func

    @property
    def project(self):
        return self._func.project

    @property
    def kb(self):
        return self.project.kb

    def analyze(self):
        ret, cache = self._check()
        if ret:
            self._analyze(cache=cache)

    def _check(self):
        """
        Check if this optimization applies to this function.

        :returns: a tuple of (does_apply, cache) where cache is a way to pass
                  information to _analyze so it does not have to be recalculated
        """
        raise NotImplementedError()

    def _analyze(self, cache=None):
        """
        Run the analysis.

        :param cache: information passed from _check so it does not have to be
                      recalculated
        :returns: None
        """
        raise NotImplementedError()


class OptimizationPass(BaseOptimizationPass):
    """
    The base class for any function-level graph optimization pass.
    """

    def __init__(
        self,
        func,
        blocks_by_addr=None,
        blocks_by_addr_and_idx=None,
        graph=None,
        variable_kb=None,
        region_identifier=None,
        reaching_definitions=None,
        **kwargs,
    ):
        super().__init__(func)
        # self._blocks is just a cache
        self._blocks_by_addr: Dict[int, Set[ailment.Block]] = blocks_by_addr
        self._blocks_by_addr_and_idx: Dict[Tuple[int, Optional[int]], ailment.Block] = blocks_by_addr_and_idx
        self._graph: Optional[networkx.DiGraph] = graph
        self._variable_kb = variable_kb
        self._ri = region_identifier
        self._rd = reaching_definitions
        self._new_block_addrs = set()

        # output
        self.out_graph: Optional[networkx.DiGraph] = None

    @property
    def blocks_by_addr(self) -> Dict[int, Set[ailment.Block]]:
        return self._blocks_by_addr

    @property
    def blocks_by_addr_and_idx(self) -> Dict[Tuple[int, Optional[int]], ailment.Block]:
        return self._blocks_by_addr_and_idx

    #
    # Util methods
    #

    def new_block_addr(self) -> int:
        """
        Return a block address that does not conflict with any existing blocks.

        :return:    The block address.
        """
        if self._new_block_addrs:
            new_addr = max(self._new_block_addrs) + 1
        else:
            new_addr = max(self.blocks_by_addr) + 2048
        self._new_block_addrs.add(new_addr)
        return new_addr

    def _get_block(self, addr, idx=None) -> Optional[ailment.Block]:
        if not self._blocks_by_addr:
            return None
        else:
            if idx is None:
                blocks = self._blocks_by_addr.get(addr, None)
            else:
                blocks = [self._blocks_by_addr_and_idx.get((addr, idx), None)]
            if not blocks:
                return None
            if len(blocks) == 1:
                return next(iter(blocks))
            raise MultipleBlocksException(
                "There are %d blocks at address %#x.%s but only one is requested." % (len(blocks), addr, idx)
            )

    def _get_blocks(self, addr, idx=None) -> Generator[ailment.Block, None, None]:
        if not self._blocks_by_addr:
            return
        else:
            if idx is None:
                blocks = self._blocks_by_addr.get(addr, None)
                if blocks is not None:
                    yield from blocks
            else:
                block = self._blocks_by_addr_and_idx.get((addr, idx), None)
                if block is not None:
                    yield block

    def _update_block(self, old_block, new_block):
        if self.out_graph is None:
            self.out_graph = self._graph  # we do not make copy here for performance reasons. we can change it if needed

        if old_block not in self.out_graph:
            return

        in_edges = list(self.out_graph.in_edges(old_block, data=True))
        out_edges = list(self.out_graph.out_edges(old_block, data=True))

        self._remove_block(old_block)
        self.out_graph.add_node(new_block)
        self._blocks_by_addr[new_block.addr].add(new_block)
        self._blocks_by_addr_and_idx[(new_block.addr, new_block.idx)] = new_block

        for src, _, data in in_edges:
            if src is old_block:
                src = new_block
            self.out_graph.add_edge(src, new_block, **data)

        for _, dst, data in out_edges:
            if dst is old_block:
                dst = new_block
            self.out_graph.add_edge(new_block, dst, **data)

    def _remove_block(self, block):
        if self.out_graph is None:
            self.out_graph = self._graph

        if block in self.out_graph:
            self.out_graph.remove_node(block)

        if block.addr in self._blocks_by_addr and block in self._blocks_by_addr[block.addr]:
            self._blocks_by_addr[block.addr].remove(block)
            del self._blocks_by_addr_and_idx[(block.addr, block.idx)]

    @staticmethod
    def _is_add(expr):
        return isinstance(expr, ailment.Expr.BinaryOp) and expr.op == "Add"

    @staticmethod
    def _is_sub(expr):
        return isinstance(expr, ailment.Expr.BinaryOp) and expr.op == "Sub"


class SequenceOptimizationPass(BaseOptimizationPass):
    """
    The base class for any sequence node optimization pass.
    """

    ARCHES = []  # strings of supported architectures
    PLATFORMS = []  # strings of supported platforms. Can be one of the following: "win32", "linux"
    STAGE: int = None  # Specifies when this optimization pass should be executed

    def __init__(self, func, seq=None, **kwargs):
        super().__init__(func)
        self.seq = seq
        self.out_seq = None



#!/usr/bin/env python3
from unittest import main, TestCase

from angr.engines.light.data import SpOffset


class TestSpOffset(TestCase):
    def test_difference_between_two_sp_offset_is_concrete(self):
        size = 8
        first_offset = SpOffset(size, 10)
        second_offset = SpOffset(size, 20)

        self.assertEqual(first_offset - second_offset, -10)


if __name__ == "__main__":
    main()



from ailment.expression import BinaryOp, BasePointerOffset, Const

from .base import PeepholeOptimizationExprBase


class BasePointerOffsetAndMask(PeepholeOptimizationExprBase):
    __slots__ = ()

    NAME = "Ptr & mask => Ptr"
    expr_classes = (BinaryOp,)  # all expressions are allowed

    def optimize(self, expr: BinaryOp, **kwargs):
        if expr.op == "And" and isinstance(expr.operands[0], BasePointerOffset) and isinstance(expr.operands[1], Const):
            # is it a mask?
            mask = expr.operands[1].value
            if mask not in {
                0xFFFE,
                0xFFFC,
                0xFFF8,
                0xFFF0,
                0xFFFF_FFFE,
                0xFFFF_FFFC,
                0xFFFF_FFF8,
                0xFFFF_FFF0,
                0xFFFF_FFFF_FFFF_FFFE,
                0xFFFF_FFFF_FFFF_FFFC,
                0xFFFF_FFFF_FFFF_FFF8,
                0xFFFF_FFFF_FFFF_FFF0,
            }:
                return None

            # we ignore this mask
            return expr.operands[0]

        return None



#!/usr/bin/env python3
__package__ = __package__ or "tests.sim"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")

addresses_fauxware = {
    "armel": 0x8524,
    "armhf": 0x104C9,  # addr+1 to force thumb
    #'i386': 0x8048524, # commenting out because of the freaking stack check
    "mips": 0x400710,
    "mipsel": 0x4006D0,
    "ppc": 0x1000054C,
    "ppc64": 0x10000698,
    "x86_64": 0x400664,
}


class TestSimulationManager(unittest.TestCase):
    def _run_fauxware(self, arch, threads):
        p = angr.Project(os.path.join(test_location, arch, "fauxware"), load_options={"auto_load_libs": False})

        pg = p.factory.simulation_manager(threads=threads)
        assert len(pg.active) == 1
        assert pg.active[0].history.depth == 0

        # step until the backdoor split occurs
        pg2 = pg.step(until=lambda lpg: len(lpg.active) > 1, step_func=lambda lpg: lpg.prune())
        assert len(pg2.active) == 2
        assert any(b"SOSNEAKY" in s for s in pg2.mp_active.posix.dumps(0).mp_items)
        assert not all(b"SOSNEAKY" in s for s in pg2.mp_active.posix.dumps(0).mp_items)

        # separate out the backdoor and normal paths
        pg3 = pg2.stash(lambda path: b"SOSNEAKY" in path.posix.dumps(0), to_stash="backdoor").move("active", "auth")
        assert len(pg3.active) == 0
        assert len(pg3.backdoor) == 1
        assert len(pg3.auth) == 1

        # step the backdoor path until it returns to main
        pg4 = pg3.step(until=lambda lpg: lpg.backdoor[0].history.jumpkinds[-1] == "Ijk_Ret", stash="backdoor")
        main_addr = pg4.backdoor[0].addr

        assert len(pg4.active) == 0
        assert len(pg4.backdoor) == 1
        assert len(pg4.auth) == 1

        # now step the real path until the real authentication paths return to the same place
        pg5 = pg4.explore(find=main_addr, num_find=2, stash="auth").move("found", "auth")

        assert len(pg5.active) == 0
        assert len(pg5.backdoor) == 1
        assert len(pg5.auth) == 2

        # now unstash everything
        pg6 = pg5.unstash(from_stash="backdoor").unstash(from_stash="auth")
        assert len(pg6.active) == 3
        assert len(pg6.backdoor) == 0
        assert len(pg6.auth) == 0

        assert len(set(pg6.mp_active.addr.mp_items)) == 1

        # now merge them!
        pg7 = pg6.merge()
        assert len(pg7.active) == 2
        assert len(pg7.backdoor) == 0
        assert len(pg7.auth) == 0

        # test selecting paths to step
        pg8 = p.factory.simulation_manager()
        pg8.step(until=lambda lpg: len(lpg.active) > 1, step_func=lambda lpg: lpg.prune().drop(stash="pruned"))
        st1, st2 = pg8.active
        pg8.step(selector_func=lambda p: p is st1, step_func=lambda lpg: lpg.prune().drop(stash="pruned"))
        assert st2 is pg8.active[1]
        assert st1 is not pg8.active[0]

        total_active = len(pg8.active)

        # test special stashes
        assert len(pg8.stashes["stashed"]) == 0
        pg8.stash(filter_func=lambda p: p is pg8.active[1], to_stash="asdf")
        assert len(pg8.stashes["stashed"]) == 0
        assert len(pg8.asdf) == 1
        assert len(pg8.active) == total_active - 1
        pg8.stash(from_stash=pg8.ALL, to_stash="fdsa")
        assert len(pg8.asdf) == 0
        assert len(pg8.active) == 0
        assert len(pg8.fdsa) == total_active
        pg8.stash(from_stash=pg8.ALL, to_stash=pg8.DROP)
        assert all(len(s) == 0 for s in pg8.stashes.values())

    def test_fauxware_armel(self):
        self._run_fauxware("armel", None)

    def test_fauxware_armhf(self):
        self._run_fauxware("armhf", None)

    def test_fauxware_mips(self):
        self._run_fauxware("mips", None)

    def test_fauxware_mipsel(self):
        self._run_fauxware("mipsel", None)

    def test_fauxware_ppc(self):
        self._run_fauxware("ppc", None)

    def test_fauxware_ppc64(self):
        self._run_fauxware("ppc64", None)

    def test_fauxware_x86_64(self):
        self._run_fauxware("x86_64", None)

    def test_find_to_middle(self):
        # Test the ability of PathGroup to execute until an instruction in the middle of a basic block
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), load_options={"auto_load_libs": False})

        pg = p.factory.simulation_manager()
        pg.explore(find=(0x4006EE,))

        assert len(pg.found) == 1
        assert pg.found[0].addr == 0x4006EE

    def test_explore_with_cfg(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), load_options={"auto_load_libs": False})

        cfg = p.analyses.CFGEmulated()

        pg = p.factory.simulation_manager()
        pg.use_technique(angr.exploration_techniques.Explorer(find=0x4006ED, cfg=cfg, num_find=3))
        pg.run()

        assert len(pg.active) == 0
        assert len(pg.avoid) == 1
        assert len(pg.found) == 2
        assert pg.found[0].addr == 0x4006ED
        assert pg.found[1].addr == 0x4006ED
        assert pg.avoid[0].addr == 0x4007C9


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import logging
import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


def run_tracker(track_mem, use_bp):
    p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
    p.analyses.CFGFast()
    main = p.kb.functions["main"]
    sp = p.arch.sp_offset
    regs = {sp}
    if use_bp:
        bp = p.arch.bp_offset
        regs.add(bp)
    sptracker = p.analyses.StackPointerTracker(main, regs, track_memory=track_mem)
    sp_result = sptracker.offset_after(0x4007D4, sp)
    if use_bp:
        bp_result = sptracker.offset_after(0x4007D4, bp)
        return sp_result, bp_result
    else:
        return sp_result


def init_tracker(track_mem):
    p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
    p.analyses.CFGFast()
    main = p.kb.functions["main"]
    sp = p.arch.sp_offset
    regs = {sp}
    sptracker = p.analyses.StackPointerTracker(main, regs, track_memory=track_mem)
    return sptracker, sp


class TestStackPointerTracker(unittest.TestCase):
    def test_stack_pointer_tracker(self):
        sp_result, bp_result = run_tracker(track_mem=True, use_bp=True)
        assert sp_result == 8
        assert bp_result == 0

    def test_stack_pointer_tracker_no_mem(self):
        sp_result, bp_result = run_tracker(track_mem=False, use_bp=True)
        assert sp_result == 8
        assert bp_result is None

    def test_stack_pointer_tracker_just_sp(self):
        sp_result = run_tracker(track_mem=False, use_bp=False)
        assert sp_result is None

    def test_stack_pointer_tracker_offset_block(self):
        sptracker, sp = init_tracker(track_mem=False)
        sp_result = sptracker.offset_after_block(0x40071D, sp)
        assert sp_result is not None
        sp_result = sptracker.offset_after_block(0x400700, sp)
        assert sp_result is None
        sp_result = sptracker.offset_before_block(0x40071D, sp)
        assert sp_result is not None
        sp_result = sptracker.offset_before_block(0x400700, sp)
        assert sp_result is None


if __name__ == "__main__":
    logging.getLogger("angr.analyses.stack_pointer_tracker").setLevel(logging.INFO)
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestFindObjectsStatic(unittest.TestCase):
    def test_object_identification_x86_64(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "cpp_classes"), auto_load_libs=False)
        object_identifier_analysis = p.analyses.StaticObjectFinder()
        possible_objects_dict = object_identifier_analysis.possible_objects
        possible_constructors = object_identifier_analysis.possible_constructors
        class_labels = []

        for possible_object in possible_objects_dict.values():
            class_labels.append(possible_object.class_name)

        assert "C" in class_labels
        assert len(possible_objects_dict) == 2
        assert len(possible_constructors) == 1
        assert 0x401512 in possible_constructors
        assert len(possible_constructors[0x401512]) == 2


if __name__ == "__main__":
    unittest.main()



# pylint:disable=unused-import
from ..models import DbLabel
from ...knowledge_plugins.labels import Labels


class LabelsSerializer:
    """
    Serialize/unserialize labels to/from a database session.
    """

    @staticmethod
    def dump(session, db_kb, labels):
        """

        :param session:
        :param DbKnowledgeBase db_kb:
        :param Labels labels:
        :return:                        None
        """

        for addr, name in labels.items():
            db_label = (
                session.query(DbLabel)
                .filter_by(
                    kb=db_kb,
                    addr=addr,
                )
                .scalar()
            )
            if db_label is not None:
                if name == db_label.name:
                    continue
                db_label.name = name
            else:
                db_label = DbLabel(
                    kb=db_kb,
                    addr=addr,
                    name=name,
                )
                session.add(db_label)

    @staticmethod
    def load(session, db_kb, kb):  # pylint:disable=unused-argument
        """

        :param session:
        :param DbKnowledgeBase db_kb:
        :param KnowledgeBase kb:
        :return:
        """

        db_labels = db_kb.labels
        labels = Labels(kb)

        for db_label in db_labels:
            labels[db_label.addr] = db_label.name

        return labels



from ..java import JavaSimProcedure


class LoadLibrary(JavaSimProcedure):
    __provides__ = (("java.lang.System", "loadLibrary(java.lang.String)"),)

    def run(self, lib):  # pylint: disable=arguments-differ,unused-argument
        pass



#!/usr/bin/env python3
# pylint:disable=missing-class-docstring,no-self-use
from unittest import TestCase, main

import claripy
from angr.storage.memory_mixins.paged_memory.pages.multi_values import MultiValues


class TestMultiValues(TestCase):
    def test_multivalues_one_slot_has_multiple_sized_bvs(self):
        mv = MultiValues(offset_to_values={0: {claripy.BVV(0, 32)}, 4: {claripy.BVV(1, 32)}, 8: {claripy.BVV(2, 32)}})
        mv.add_value(4, claripy.BVV(0x1338133813371337, 64))

        assert len(mv._values) == 3
        assert len(mv._values[4]) == 2
        assert mv._values[4] == {claripy.BVV(1, 32), claripy.BVV(0x13381338, 32)}
        assert mv._values[8] == {claripy.BVV(2, 32), claripy.BVV(0x13371337, 32)}

        mv.add_value(5, claripy.BVV(0xCC, 8))
        assert len(mv._values) == 5  # 0, 4, 5, 6, 8
        assert mv._values[5] == {claripy.BVV(0xCC, 8), claripy.BVV(0x38, 8), claripy.BVV(0, 8)}
        assert mv._values[6] == {claripy.BVV(1, 16), claripy.BVV(0x1338, 16)}


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")
arches = {"x86_64"}


class TestDdgGlobalVarDependencies(unittest.TestCase):
    def test_ddg_global_var_dependencies(self):
        for arch in arches:
            self.run_ddg_global_var_dependencies(arch)

    def run_ddg_global_var_dependencies(self, arch):
        test_file = os.path.join(test_location, arch, "ddg_global_var_dependencies")
        proj = angr.Project(test_file, auto_load_libs=False)
        cfg = proj.analyses.CFGEmulated(
            context_sensitivity_level=2,
            keep_state=True,
            state_add_options=angr.sim_options.refs,
        )
        ddg = proj.analyses.DDG(cfg)
        main_func = cfg.functions.function(name="main")

        target_block_addr = main_func.ret_sites[0].addr
        target_block = proj.factory.block(addr=target_block_addr)
        tgt_stmt_idx, tgt_stmt = self.get_target_stmt(proj, target_block)
        assert tgt_stmt_idx is not None
        buf_addr = tgt_stmt.data.addr.con.value
        tgt_ddg_node = self.get_ddg_node(ddg, target_block_addr, tgt_stmt_idx)
        assert tgt_ddg_node is not None

        # Whether the target depends on the statement assigning 'b' to the global variable
        has_correct_dependency = False
        for pred in ddg.get_predecessors(tgt_ddg_node):
            pred_block = proj.factory.block(addr=pred.block_addr)
            stmt = pred_block.vex.statements[pred.stmt_idx]
            has_correct_dependency |= self.check_dependency(stmt, buf_addr, ord("b"))

            # If the target depends on the statement assigning 'a' to the global variable, it is underconstrained (
            # this assignment should be overwritten by the 'b' assignment)
            assert not self.check_dependency(
                stmt, buf_addr, ord("a")
            ), "Target statement has incorrect dependency (DDG is underconstrained)"
        assert has_correct_dependency, "Target statement does not have correct dependency (DDG is overconstrained)"

    def check_dependency(self, stmt, addr, const):
        # Check if we are storing a constant to a variable with constant address
        if stmt.tag == "Ist_Store" and stmt.addr.tag == "Iex_Const" and stmt.data.tag == "Iex_Const":
            # Check if we are storing the specified constant to the specified variable address
            if stmt.addr.con.value == addr and stmt.data.con.value == const:
                return True

        return False

    def get_ddg_node(self, ddg, block_addr, stmt_idx):
        for node in ddg.graph.nodes:
            if node.block_addr == block_addr and node.stmt_idx == stmt_idx:
                return node
        return None

    def get_target_stmt(self, proj, block):
        for i, stmt in enumerate(block.vex.statements):
            # We're looking for the instruction that loads a constant memory address into a temporary variable
            if stmt.tag == "Ist_WrTmp" and stmt.data.tag == "Iex_Load" and stmt.data.addr.tag == "Iex_Const":
                addr = stmt.data.addr.con.value
                section = proj.loader.main_object.find_section_containing(addr)
                # Confirm the memory address is in the uninitialized data section
                if section.name == ".bss":
                    return i, stmt
        return None, None


if __name__ == "__main__":
    unittest.main()



from ailment.expression import BinaryOp, Const

from .base import PeepholeOptimizationExprBase


class ASubADiv(PeepholeOptimizationExprBase):
    __slots__ = ()

    NAME = "a - a / N => a * (N - 1) / N"
    expr_classes = (BinaryOp,)

    def optimize(self, expr: BinaryOp, **kwargs):
        if expr.op == "Sub" and len(expr.operands) == 2:
            expr0, expr1 = expr.operands
            if isinstance(expr1, BinaryOp) and expr1.op == "Div" and isinstance(expr1.operands[1], Const):
                a = expr0
                if expr1.operands[0].likes(a):
                    N = expr1.operands[1].value
                    mul = BinaryOp(expr.idx, "Mul", [a, Const(None, None, N - 1, expr.bits)], False, **expr.tags)
                    div = BinaryOp(
                        expr1.idx, "Div", [mul, Const(None, None, N, expr.bits, **expr1.tags)], False, **expr1.tags
                    )
                    return div

        return None



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("slcext.dll")
prototypes = \
    {
        #
        'SLActivateProduct': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"cbSize": SimTypeInt(signed=False, label="UInt32"), "type": SimTypeInt(signed=False, label="SL_ACTIVATION_TYPE")}, name="SL_ACTIVATION_INFO_HEADER", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypeInt(signed=True, label="Int32"), arg_names=["hSLC", "pProductSkuId", "cbAppSpecificData", "pvAppSpecificData", "pActivationInfo", "pwszProxyServer", "wProxyPort"]),
        #
        'SLGetServerStatus': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwszServerURL", "pwszAcquisitionType", "pwszProxyServer", "wProxyPort", "phrStatus"]),
        #
        'SLAcquireGenuineTicket': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppTicketBlob", "pcbTicketBlob", "pwszTemplateId", "pwszServerUrl", "pwszClientToken"]),
        #
        'SLGetReferralInformation': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="SLREFERRALTYPE"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hSLC", "eReferralType", "pSkuOrAppId", "pwszValueName", "ppwszValue"]),
    }

lib.set_prototypes(prototypes)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.procedures.libc"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ...common import bin_location

TARGET_APP = os.path.join(bin_location, "tests", "x86_64", "fgets")

p = angr.Project(TARGET_APP, auto_load_libs=False)

find_normal = p.loader.find_symbol("find_normal").rebased_addr
find_exact = p.loader.find_symbol("find_exact").rebased_addr
find_eof = p.loader.find_symbol("find_eof").rebased_addr
find_impossible = p.loader.find_symbol("find_impossible").rebased_addr


class TestFgets(unittest.TestCase):
    def _testfind(self, addr, failmsg):
        e = p.factory.entry_state()
        e.options.add(angr.sim_options.SHORT_READS)
        e.options.add(angr.sim_options.SYMBOL_FILL_UNCONSTRAINED_MEMORY)
        s = p.factory.simgr(e)
        r = s.explore(find=addr)
        assert len(r.found) > 0, failmsg
        return r.found[0].posix.dumps(0)

    def _testnotfind(self, addr, failmsg):
        e = p.factory.entry_state()
        e.options.add(angr.sim_options.SHORT_READS)
        e.options.add(angr.sim_options.SYMBOL_FILL_UNCONSTRAINED_MEMORY)
        s = p.factory.simgr(e)
        r = s.explore(find=addr)
        assert len(r.found) == 0, failmsg

    def test_normal(self):
        answer = self._testfind(find_normal, "Normal Failure!")
        assert answer == b"normal\n"

    def test_exact(self):
        answer = self._testfind(find_exact, "Exact Failure!")
        assert answer.endswith(b"0123456789")

    def test_impossible(self):
        self._testnotfind(find_impossible, "Impossible Failure!")


if __name__ == "__main__":
    unittest.main()
